class ActorMessage:
	def __init__(self):
		pass

class TraceableActorMessage(ActorMessage):
	def __init__(self, tracking):
		ActorMessage.__init__(self)
		self.__tracking = tracking

	def tracking(self):
		return self.__tracking

class TraceableActorFailureMessage(TraceableActorMessage):
	def __init__(self, tracking, reason, source_message=None):
		TraceableActorMessage.__init__(self,tracking)
		self.__reason=reason
		self.__source_message=source_message

	def reason(self):
		return self.__reason

	def source_message(self):
		return self.__source_message		

class ActorFailureMessage(ActorMessage):
	def __init__(self, reason, source_message=None):
		ActorMessage.__init__(self)
		self.__reason=reason
		self.__source_message=source_message

	def reason(self):
		return self.__reason

	def source_message(self):
		return self.__source_message

class TrackingActorFailureMessage(ActorFailureMessage):
	def __init__(self, tracking, reason, source_message=None):
		ActorFailureMessage.__init__(self, reason, source_message)
		self.__tracking=tracking

	def tracking(self):
		return self.__tracking

class PoisonPill(ActorMessage):
	def __init__(self):
		ActorMessage.__init__(self)


class Terminated(ActorMessage):
	def __init__(self, actor_id):
		ActorMessage.__init__(self)
		self.__actor_id=actor_id

	def actor_id(self):
		return self.__actor_id



import re
import os

class MedianameEvaluator():
	def __init__(self, mediapath):
		self.__mediapath=mediapath

	@staticmethod
	def create_from_path(mediapath):
		return MedianameEvaluator(os.path.basename(mediapath))

	def evaluate(self):

		track_number = TrackNumberGuesser.make_a_guess(self.__mediapath)

		result = MedianameEvaluationResult()
		result.track_number = track_number

		return result

class MedianameEvaluationResult():
	
	def __init__(self):
		self.__track_number=None

	@property
	def track_number(self):
		return self.__track_number

	@track_number.setter
	def track_number(self, value):
		self.__track_number = value

class MediaTagGuesser():

	@staticmethod
	def guess_value(value):
		raise NotImplemented()



class TrackNumberGuesser(MediaTagGuesser):

	@staticmethod
	def make_a_guess(medianame):
		m = re.search('(?<!\.mp)([0-9]{1,3})', medianame)
		
		if m is not None:
			return m.group(0)
		else:
			return None


from longtang.actors import messages

class KickOffMediaProcessing(messages.ActorMessage):
	def __init__(self):
		messages.ActorMessage.__init__(self)

class MediaProcessingFinished(messages.ActorMessage):
	def __init__(self, summary):
		messages.ActorMessage.__init__(self)
		self.__summary=summary

	def summary(self):
		return self.__summary


import gevent
import shutil
import tempfile
import os
import sys

from cStringIO import StringIO
from longtang.actors import actors, messages
from longtang.system import system, domain as sys_domain

class TestActor(actors.Actor):

	def __init__(self, config, system_ref, unique_id):
		actors.Actor.__init__(self, system_ref, unique_id)
		self.messages={}
		self.__actor_ref = None
		self.__config=config

	def pre_setup(self):

		if not self.__config.use_factory():
			self.__actor_ref=self.context().from_type(self.__config.actor_type,self.__config.unique_id)
		else:
			self.__actor_ref=self.context().with_factory(self.__config.factory_type,self.__config.unique_id)

	def __force_termination(self):
		self.logger().info('Moratorium expired, termination will be forced....')
		self.context().get_by_id(self.__config.unique_id).tell(messages.PoisonPill(), self)

	def receive(self, message):
		msg_type = message.__class__

		print 'Tester received message....{0}'.format(message)

		if msg_type in self.messages:
			self.messages[msg_type].append(message)
		else:
			self.messages[msg_type] = [message]

		if isinstance(message, messages.Terminated):
			self._silently_end()

		if self.__config.termination_type is not None:
			if isinstance(message, self.__config.termination_type):
				print 'Termination message type received....terminating'
				self.context().get_by_id(self.__config.unique_id).tell(messages.PoisonPill(), self)

	def test(self,message, sender=None):

		print 'Tester.....{0}'.format(message)

		actor_ref = self.context().get_by_id(self.__config.unique_id)

		actor_ref.tell(message, self)

		if self.__config.termination_type is None:
			actor_ref.tell(messages.PoisonPill(), self)
		else:
			moratorium = gevent.Greenlet(self.__force_termination)
			moratorium.start_later(self.__config.termination_moratorium)

		#TODO Should be able to work with system
		gevent.joinall([actor_ref._instance()])

	def inspector(self):
		return MessageSetInspector(self.messages)

	class TestActorContext():

		def __init__(self, current_message, test_actor_ref):
			self.current_message = current_message
			self.test_actor_ref = test_actor_ref

		def current_message(self):
			return self.current_message


class TestActorBuilder():
	def __init__(self):
		self.actor_type=None
		self.__factory_type=None
		self.__unique_id='random-id-to-be-generated'
		self.__termination_message_type=None
		self.__termination_moratorium=1 #Seconds
		self.__terminate_system=True

	def terminate_system(self, value):
		self.__terminate_system = value
		return self

	def withType(self, actor_type):
		self.actor_type = actor_type
		return self

	def withFactory(self, factory_type):
		self.__factory_type = factory_type
		return self		

	def terminationType(self, message_type):
		self.__termination_message_type = message_type
		return self

	def terminationMoratorium(self, time):
		self.__termination_moratorium=time
		return self

	def withId(self, unique_id):
		self.__unique_id=unique_id
		return self

	def build(self):
		
		actor_system = system.ActorSystem()

		config = TestActorConfiguration()

		config.actor_type = self.actor_type
		config.factory_type = self.__factory_type
		config.unique_id = self.__unique_id
		config.termination_type = self.__termination_message_type
		config.termination_moratorium = self.__termination_moratorium

		return TestActorRef(actor_system, actor_system.with_factory(TestActorFactory(config), 'testutils-test-actor'), self.__terminate_system)

class TestActorRef(sys_domain.ActorRef):
	def __init__(self, system, actor_ref, terminate=True):
		sys_domain.ActorRef.__init__(self, actor_ref._instance(), actor_ref.parent())
		self.__system=system
		self.__terminate=terminate

	def tell(self,message, sender=None):
		self._instance().test(message, sender)
		
		if self.__terminate is True:
			self.__system.shutdown()

	def inspector(self):
		return self._instance().inspector()

	def system(self):
		return self.__system

class TestActorConfiguration:

	def __init__(self):
		self.__termination_type = None
		self.__termination_moratorium = None
		self.__unique_id=None
		self.__actor_type=None
		self.__factory_type=None

	@property
	def factory_type(self):
		return self.__factory_type

	@factory_type.setter
	def factory_type(self, value):
		self.__factory_type=value

	@property
	def actor_type(self):
		return self.__actor_type

	@actor_type.setter
	def actor_type(self, value):
		self.__actor_type=value

	@property
	def unique_id(self):
		return self.__unique_id

	@unique_id.setter
	def unique_id(self, value):
		self.__unique_id=value

	@property
	def termination_type(self):
		return self.__termination_type

	@termination_type.setter
	def termination_type(self, value):
		self.__termination_type = value

	@property
	def termination_moratorium(self):
		return self.__termination_moratorium

	@termination_moratorium.setter
	def termination_moratorium(self, value):
		self.__termination_moratorium = value

	def use_factory(self):
		return True if self.__actor_type is None and self.factory_type is not None else False

class TestActorFactory(sys_domain.ActorFactory):
	def __init__(self, config):
		self.__config = config

	def create(self, unique_id, system_ref):
		return TestActor(self.__config, system_ref, unique_id)

class MessageSetInspector:
	def __init__(self, messages):
		self.messages = messages

	def num_instances(self,msg_type=None):

		def map_elements(msg_list):
			return len(msg_list)

		def reduce_elements(accum, value):
			return accum + value

		message_group = self.messages.itervalues()

		if msg_type is not None:
			try:
				message_group = [self.messages[msg_type]]
			except KeyError:
				message_group = [[]]

		return reduce(reduce_elements,map(map_elements, message_group))

	def get_first(self, msg_type):
		try:
			return self.messages[msg_type][0]
		except KeyError, e:
			return None
		

def current_dirpath(source_file):
	return os.path.dirname(os.path.realpath(source_file))

def copy_to_tmp(source, prefix='longtang_', suffix='.tmp', dir=None):

	tmp_file=tempfile.NamedTemporaryFile('w+b',-1,suffix, prefix, delete=False, dir=dir)

	print 'Temp path....{0}'.format(tmp_file.name)

	shutil.copy2(source,tmp_file.name)

	return tmp_file.name

def create_tmp_dir(prefix='tmp_', suffix='', target_dir=None):

	return tempfile.mkdtemp(suffix, prefix, target_dir)

def remove_tmp_dir(path):
	shutil.rmtree(path,True)

def remove_dir_content(path):
	for root, dirs, files in os.walk(path):
		for f in files:
			os.unlink(os.path.join(root, f))
		for d in dirs:
			shutil.rmtree(os.path.join(root, d))

class StdoutSniffer():

	def __init__(self):
		self.__backup=None
		self.__value=None

	def start(self):
		# setup the environment
		self.__backup = sys.stdout

		# ####
		sys.stdout = StringIO()     # capture output

	def stop(self):
		self.__value = sys.stdout.getvalue() # release output
		# ####

		sys.stdout.close()  # close the stream 
		sys.stdout = self.__backup # restore original stdout

	def value(self):
		return self.__value      



#https://github.com/hamcrest/PyHamcrest
#http://packages.python.org/PyHamcrest/

import gevent
import unittest
import os

from hamcrest import *
from longtang.system import system, domain as sys_domain
from longtang.actors import testutils
from longtang.actors.tracking import facade, tracking
from longtang.common import mediatracking

class TestTrackingActorFacade(unittest.TestCase):
	def test_create_tracking(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'mediatracking-actor')

		tracking_facade = facade.MediaTrackingFacade.from_system(sys_domain.SystemRef(actor_system))

		tracking_id = tracking_facade.create_tracking('/dummy.mp3')

		assert_that(tracking_id, is_(not_none()),'Tracking identifier is empty')

		actor_system.shutdown()	

	def test_lookup(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'mediatracking-actor')

		tracking_facade = facade.MediaTrackingFacade.from_system(sys_domain.SystemRef(actor_system))

		tracking_id = tracking_facade.create_tracking('/dummy.mp3')
		tracking_info = tracking_facade.lookup(tracking_id)

		assert_that(tracking_info, is_(not_none()),'Tracking information is empty')
		assert_that(tracking_info.mediafile.sourcepath, is_('/dummy.mp3'),'Tracking sourcepath information is not right')

	def test_notify(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'mediatracking-actor')

		tracking_facade = facade.MediaTrackingFacade.from_system(sys_domain.SystemRef(actor_system))		

		tracking_id = tracking_facade.create_tracking('/dummy.mp3')

		event = mediatracking.MediaFileTrackingEvent('NONALTERING', 'This is a non-altering event')

		tracking_facade.notify(tracking_id, event)

		tracking_info = tracking_facade.lookup(tracking_id)

		assert_that(tracking_info.tracking_events.of_type('NONALTERING').is_empty(), is_(False),'Tracking events of type NONALTERING are empty')
		assert_that(tracking_info.tracking_events.of_type('NONALTERING').get(0), is_(not_none()),'Tracked events does not exist')



#http://docs.python.org/2/library/unittest.html#unittest.TestCase.assertEqual

import unittest
import os
import gevent

from hamcrest import *
from longtang.actors import testutils,actors,domain, messages
from longtang.system import system, domain as sys_domain

class TestActorSystem(unittest.TestCase):

    def test_actor_creation_by_type(self):

        actor_system = system.ActorSystem()
        dummy_actor_ref = actor_system.from_type(DummyActor,'dummy-actor')

        self.assertIsNotNone(dummy_actor_ref, 'Actor has not been created')
        self.assertIsNotNone(dummy_actor_ref.context(), 'System reference does not exist within actor instance')
        self.assertNotEqual(dummy_actor_ref.context(), actor_system, 'System reference is directly passed to the actor!')
        self.assertTrue(isinstance(dummy_actor_ref, sys_domain.ActorRef), 'Actor reference is not of the expected type')
        self.assertTrue(isinstance(dummy_actor_ref.parent(), sys_domain.ActorRef), 'Parent actor reference is not of the expected type')
        self.assertTrue(isinstance(dummy_actor_ref.context(),domain.SystemRefWrapper), 'System information within actor is not of the right type')        
        self.assertIsNotNone(dummy_actor_ref.context().get_by_id('dummy-actor'), 'Actor does not exist within context')
        self.assertIsNotNone(actor_system.find_by_id('dummy-actor'), 'Actor does not exist within context')

        actor_system.shutdown()

    def test_actor_creation_with_factory(self):

        actor_system = system.ActorSystem()
        dummy_actor_ref = actor_system.with_factory(DummyFactory(),'dummy-factory-actor')

        self.assertIsNotNone(dummy_actor_ref, 'Actor has not been created')
        self.assertIsNotNone(dummy_actor_ref.context(), 'System reference does not exist within actor instance')
        self.assertNotEqual(dummy_actor_ref.context(), actor_system, 'System reference is directly passed to the actor!')
        self.assertTrue(isinstance(dummy_actor_ref.context(),domain.SystemRefWrapper), 'System information within actor is not of the right type')
        self.assertTrue(isinstance(dummy_actor_ref, sys_domain.ActorRef), 'Actor reference is not of the expected type')
        self.assertTrue(isinstance(dummy_actor_ref.parent(), sys_domain.ActorRef), 'Parent actor reference is not of the expected type')        
        self.assertIsNotNone(dummy_actor_ref.context().get_by_id('dummy-factory-actor'), 'Actor does not exist within context')
        self.assertIsNotNone(actor_system.find_by_id('dummy-factory-actor'), 'Actor does not exist within context')

        actor_system.shutdown()

    def test_child_creation(self):

        actor_system = system.ActorSystem()
        main_actor = actor_system.from_type(DummyActor,'main-actor')

        assert_that(actor_system.find_by_id('main-actor'), is_not(None), 'Main actor does not exist within system')
        assert_that(actor_system.find_by_id('dummy-child-actor'), is_not(None), 'Child actor does not exist within system')
        assert_that(actor_system.find_by_id('dummy-child-actor'), is_(instance_of(sys_domain.ActorRef)), 'Child actor is not of the right type')
        assert_that(actor_system.find_by_id('dummy-child-actor')._instance(), is_(instance_of(DummyChildActor)), 'Child actor is not of the right type')
        assert_that(actor_system.find_by_id('dummy-child-actor').parent(), is_(equal_to(actor_system.find_by_id('main-actor'))),'Parent actor is not correctly assigned')

        #TODO Add test related to child methods

        actor_system.shutdown()

    def test_child_management(self):

        actor_system = system.ActorSystem()
        main_actor = actor_system.from_type(DummyActor,'main-actor')

        assert_that(actor_system.find_by_id('main-actor').status(), is_(equal_to(domain.ActorStatus.READY)), 'Main actor status has not been initialized as expected')
        assert_that(actor_system.find_by_id('dummy-child-actor').status(), is_(equal_to(domain.ActorStatus.READY)), 'Child actor status has not been initialized as expected')

        main_actor.tell(TerminateYourself())

        actor_system.wait_for(main_actor)

        assert_that(main_actor.context().child('dummy-child-actor').status(), is_(equal_to(domain.ActorStatus.TERMINATED)), 'Child actor has not been terminated as expected')
        assert_that(main_actor.status(), is_(equal_to(domain.ActorStatus.TERMINATED)), 'Child actor has not been terminated as expected')
        assert_that(actor_system.find_by_id('main-actor').status(), is_(equal_to(domain.ActorStatus.TERMINATED)), 'Child actor has not been terminated as expected')
        assert_that(actor_system.find_by_id('dummy-child-actor').status(), is_(equal_to(domain.ActorStatus.TERMINATED)), 'Child actor has not been terminated as expected')

        actor_system.shutdown()

    def test_marooned_messages(self):

        actor_system = system.ActorSystem()
        main_actor = actor_system.from_type(MaroonedMessageActor,'main-actor')

        main_actor.tell(MaroonedMessage())
        main_actor.tell(TerminateYourself())

        actor_system.wait_for(main_actor)

        assert_that(actor_system.marooned_messages().reported_by('main-actor'), is_(not_none()),'Main actor marooned messages list is none')
        assert_that(actor_system.marooned_messages().reported_by('main-actor').size(), is_(equal_to(1)),'Main actor marooned messages list has no elements')
        assert_that(actor_system.marooned_messages().reported_by('main-actor').get(0), is_(instance_of(MaroonedMessage)),'Main actor marooned message is not the expected one')
        assert_that(actor_system.marooned_messages().all(), is_(not_none()),'Global marooned messages list is none')
        assert_that(actor_system.marooned_messages().all().size(), is_(equal_to(1)),'Global marooned messages list is empty')

        actor_system.shutdown()


class DummyActor(actors.Actor):

    def pre_setup(self):
        
        self.__dummy_child_ref = self.context().from_type(DummyChildActor,'dummy-child-actor')

    def receive(self, message):
        if isinstance(message, TerminateYourself):
            gevent.sleep(0.5)
            self.myself().tell(messages.PoisonPill())

class DummyChildActor(actors.Actor):

    def receive(self, message):
        pass

class DummyFactoryActor(actors.Actor):

    def receive(self, message):
        pass

class DummyFactory(sys_domain.ActorFactory):

    def create(self, unique_id, system_ref):
        return DummyFactoryActor(system_ref,unique_id)

class MaroonedMessageActor(actors.Actor):

    def receive(self, message):
        if isinstance(message, TerminateYourself):
            gevent.sleep(0.5)
            self.myself().tell(messages.PoisonPill())
        else:        
            self.notify_marooned_message(message) 


class MaroonedMessage(messages.ActorMessage):
    pass

class TerminateYourself(messages.ActorMessage):
    pass

if __name__ == '__main__':
    unittest.main()


import logger

from longtang.system import domain

class LoggerActorFactory(domain.ActorFactory):
	def __init__(self, level):
		self.__level = level

	def create(self, unique_id, system_ref):
		return logger.LoggerActor(self.__level, system_ref, unique_id)


from longtang.common import domain as common_domain

class AcoustidResponseWrapper():
	def __init__(self, response):
		self.__response=response

	def album(self):
		if 'results' in self.__response:
			if len(self.__response['results']) > 0:
				if 'recordings' in self.__response['results'][0]:
					if len(self.__response['results'][0]['recordings']) > 0:
						if 'releasegroups' in self.__response['results'][0]['recordings'][0]:
							if len(self.__response['results'][0]['recordings'][0]['releasegroups']) > 0:
								return self.__response['results'][0]['recordings'][0]['releasegroups'][0]['title']

		return None

	def artist(self):
		if 'results' in self.__response:
			if len(self.__response['results']) > 0:
				if 'recordings' in self.__response['results'][0]:
					if len(self.__response['results'][0]['recordings']) > 0:
						if 'artists' in self.__response['results'][0]['recordings'][0]:
							if len(self.__response['results'][0]['recordings'][0]['artists']) > 0:
								return self.__response['results'][0]['recordings'][0]['artists'][0]['name']

		return None

	def title(self):
		if 'results' in self.__response:		
			if len(self.__response['results']) > 0:
				if 'recordings' in self.__response['results'][0]:
					if len(self.__response['results'][0]['recordings']) > 0:
						return self.__response['results'][0]['recordings'][0]['title']

		return None

	def found(self):

		return self.album() is not None and self.artist() is not None and self.title() is not None

	def successful(self):
		return self.__response['status'] == 'ok'

	def extract_metadata(self):
		builder = common_domain.FileMetadataBuilder()

		return builder.artist(self.artist()) \
						.album(self.album()) \
						.title(self.title()).build()


# -*- coding: utf-8 -*-

import gevent
import unittest
import os

from hamcrest import *
from longtang.actors.supervisors.main import main, messages, factory
from longtang.actors.supervisors.flowconductor import flowconductor
from longtang.actors.inbound import filepoller
from longtang.actors import testutils
from longtang.system import system, exceptions
from longtang.config import configuration
from longtang.common import interactions
from longtang.actors.media.id3tag import domain

class TestMainActor(unittest.TestCase):

	def setUp(cls):
		global base_from_test_dir
		global base_to_test_dir

		base_from_test_dir = testutils.create_tmp_dir(suffix='maintest_from')
		base_to_test_dir = testutils.create_tmp_dir(suffix='maintest_to')

	def tearDown(cls):
		testutils.remove_tmp_dir(base_from_test_dir) 
		testutils.remove_tmp_dir(base_to_test_dir)

	def test_creation(self):

		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')

		config_builder = configuration.LongTangConfigurationBuilder()
		config = config_builder.read_from(base_from_test_dir).generate_at(base_to_test_dir).build()

		actor_system = system.ActorSystem()
		main_actor = actor_system.with_factory(factory.MainSupervisorFactory(config),'main-actor')
		
		try:		
			assert_that(actor_system.find_by_id('main-actor'), is_not(None), 'Main actor does not exist within system')

			assert_that(actor_system.find_by_id('file-poller-actor'), is_not(None), 'File poller actor does not exist within system')
			assert_that(actor_system.find_by_id('flowconductor-actor'), is_not(None), 'Flowconductor actor actor does not exist within system')

			assert_that(actor_system.find_by_id('file-poller-actor').parent(), is_(equal_to(actor_system.find_by_id('main-actor'))),'Parent actor is not correctly assigned')
			assert_that(actor_system.find_by_id('flowconductor-actor').parent(), is_(equal_to(actor_system.find_by_id('main-actor'))),'Parent actor is not correctly assigned')

			assert_that(actor_system.find_by_id('file-poller-actor')._instance(), is_(instance_of(filepoller.FilePoller)), 'File poller actor is not of the right type')
			assert_that(actor_system.find_by_id('flowconductor-actor')._instance(), is_(instance_of(flowconductor.FlowConductorSupervisor)), 'Flow conductor actor is not of the right type')

		except exceptions.ActorNotFound as e:
			self.fail(str(e))

		actor_system.shutdown()

	def test_single_file_processing_with_tags(self):
		
		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')

		source_file_with_id3tag = testutils.copy_to_tmp(source=os.path.join(data_dir, 'with_id3taginfo.mp3'),suffix='.tmp.mp3',dir=base_from_test_dir)

		config_builder = configuration.LongTangConfigurationBuilder()
		config = config_builder.read_from(base_from_test_dir).generate_at(base_to_test_dir).build()

		actor_system = system.ActorSystem()
		main_ref = actor_system.with_factory(factory.MainSupervisorFactory(config),'main-actor')

		interactions.InteractionPatterns.ask(actor_system, main_ref, messages.KickOffMediaProcessing())		

		full_dir_path=os.path.join(base_to_test_dir, 'Bon Jovi', 'New Jersey')
		full_file_path=os.path.join(full_dir_path,'08 - Ride Cowboy Ride.mp3')

		assert_that(os.path.isdir(full_dir_path), is_(equal_to(True)), 'Artist or album directory does not exist or full name is not right')
		assert_that(os.path.isfile(full_file_path), is_(equal_to(True)),'Media file in target path does not exist')

		#id3tag
		reader = domain.ID3TagReaderFactory.createFromType(full_file_path)

		assert_that(reader.artist(), is_(equal_to('Bon Jovi')), 'id3tag content for artist does not match')
		assert_that(reader.title(), is_(equal_to('Ride Cowboy Ride')), 'id3tag content for title does not match')
		assert_that(reader.album(), is_(equal_to('New Jersey')), 'id3tag content for album does not match')
		assert_that(reader.track_number(), is_(equal_to('8')), 'id3tag content for track number does not match')

	def test_multiple_file_encoding_processing_with_tags(self):
		
		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data','multiple_encodings')

		#source_file_with_id3tag = testutils.copy_to_tmp(source=os.path.join(data_dir, 'with_id3taginfo.mp3'),suffix='.tmp.mp3',dir=base_from_test_dir)

		config_builder = configuration.LongTangConfigurationBuilder()
		config = config_builder.read_from(data_dir).generate_at(base_to_test_dir).build()

		actor_system = system.ActorSystem()
		main_ref = actor_system.with_factory(factory.MainSupervisorFactory(config),'main-actor')

		interactions.InteractionPatterns.ask(actor_system, main_ref, messages.KickOffMediaProcessing())	

		#full_dir_path=os.path.join(base_to_test_dir, 'Bon Jovi', 'New Jersey')
		#full_file_path=os.path.join(full_dir_path,'08 - Ride Cowboy Ride.mp3')

		#assert_that(os.path.isdir(full_dir_path), is_(equal_to(True)), 'Artist or album directory does not exist or full name is not right')
		#assert_that(os.path.isfile(full_file_path), is_(equal_to(True)),'Media file in target path does not exist')

		#id3tag
		#reader = domain.ID3TagReaderFactory.createFromType(full_file_path)

		#assert_that(reader.artist(), is_(equal_to('Bon Jovi')), 'id3tag content for artist does not match')
		#assert_that(reader.title(), is_(equal_to('Ride Cowboy Ride')), 'id3tag content for title does not match')
		#assert_that(reader.album(), is_(equal_to('New Jersey')), 'id3tag content for album does not match')
		#assert_that(reader.track_number(), is_(equal_to('8')), 'id3tag content for track number does not match')		

	def test_multiple_file_processing_with_tags(self): 
		
		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data', 'multiple')

		config_builder = configuration.LongTangConfigurationBuilder()
		config = config_builder.read_from(data_dir).generate_at(base_to_test_dir).build()

		actor_system = system.ActorSystem()
		main_ref = actor_system.with_factory(factory.MainSupervisorFactory(config),'main-actor')

		interactions.InteractionPatterns.ask(actor_system, main_ref, messages.KickOffMediaProcessing()) 

		full_dir_path_source1=os.path.join(base_to_test_dir, 'The Aristocrats', 'Boing, We\'ll Do It Live!')
		full_file_path_source1=os.path.join(full_dir_path_source1,'08 - A Very Metal Introduction.mp3')

		assert_that(os.path.isdir(full_dir_path_source1), is_(equal_to(True)), 'Artist or album directory does not exist or full name is not right')
		assert_that(os.path.isfile(full_file_path_source1), is_(equal_to(True)),'Media file in target path does not exist')

		reader = domain.ID3TagReaderFactory.createFromType(full_file_path_source1)

		assert_that(reader.artist(), is_(equal_to('The Aristocrats')), 'id3tag content for artist does not match')
		assert_that(reader.title(), is_(equal_to('A Very Metal Introduction')), 'id3tag content for title does not match')
		assert_that(reader.album(), is_(equal_to('Boing, We\'ll Do It Live!')), 'id3tag content for album does not match')
		assert_that(reader.track_number(), is_(equal_to('8')), 'id3tag content for track number does not match')

		full_dir_path_source2=os.path.join(base_to_test_dir, 'Carla Bruni', 'Quelqu\'un M\'a Dit')
		full_file_path_source2=os.path.join(full_dir_path_source2,'02 - La Dernière Minute.mp3')

		assert_that(os.path.isdir(full_dir_path_source2), is_(equal_to(True)), 'Artist or album directory does not exist or full name is not right')
		assert_that(os.path.isfile(full_file_path_source2), is_(equal_to(True)),'Media file in target path does not exist')			

		reader = domain.ID3TagReaderFactory.createFromType(full_file_path_source2)

		assert_that(reader.artist(), is_(equal_to(u'Carla Bruni')), 'id3tag content for artist does not match')
		assert_that(reader.title(), is_(equal_to(u'La Dernière Minute')), 'id3tag content for title does not match')
		assert_that(reader.album(), is_(equal_to(u'Quelqu\'un M\'a Dit')), 'id3tag content for album does not match')
		assert_that(reader.track_number(), is_(equal_to('2')), 'id3tag content for track number does not match')

		full_dir_path_source3=os.path.join(base_to_test_dir, 'Big Cock', 'Year Of The Cock')
		full_file_path_source3=os.path.join(full_dir_path_source3,'13 - Thank You, Good Night.mp3')

		assert_that(os.path.isdir(full_dir_path_source3), is_(equal_to(True)), 'Artist or album directory does not exist or full name is not right')
		assert_that(os.path.isfile(full_file_path_source3), is_(equal_to(True)),'Media file in target path does not exist')	

		reader = domain.ID3TagReaderFactory.createFromType(full_file_path_source3)

		assert_that(reader.artist(), is_(equal_to('Big Cock')), 'id3tag content for artist does not match')
		assert_that(reader.title(), is_(equal_to('Thank You, Good Night')), 'id3tag content for title does not match')
		assert_that(reader.album(), is_(equal_to('Year Of The Cock')), 'id3tag content for album does not match')
		assert_that(reader.track_number(), is_(equal_to('13')), 'id3tag content for track number does not match') 

	def test_summary_success(self):
		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data', 'multiple')

		config_builder = configuration.LongTangConfigurationBuilder()
		config = config_builder.read_from(data_dir).generate_at(base_to_test_dir).build()

		actor_system = system.ActorSystem()
		main_ref = actor_system.with_factory(factory.MainSupervisorFactory(config),'main-actor')

		summary = interactions.InteractionPatterns.ask(actor_system, main_ref, messages.KickOffMediaProcessing()).summary() 

		assert_that(summary.total(), equal_to(3), 'Amount of processed files is wrong')
		assert_that(summary.total_failures(), equal_to(0), 'Amount of failed files is wrong')
		assert_that(summary.total_successes(), equal_to(3), 'Amount of successed files is wrong')

	def test_summary_failure(self):

		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')

		source_file_with_id3tag = testutils.copy_to_tmp(source=os.path.join(data_dir, 'withouttaginfo.mp3'),suffix='.tmp.mp3',dir=base_from_test_dir)

		config_builder = configuration.LongTangConfigurationBuilder()
		config = config_builder.read_from(base_from_test_dir).generate_at(base_to_test_dir).build()

		actor_system = system.ActorSystem()
		main_ref = actor_system.with_factory(factory.MainSupervisorFactory(config),'main-actor')

		summary = interactions.InteractionPatterns.ask(actor_system, main_ref, messages.KickOffMediaProcessing()).summary() 

		assert_that(summary.total(), equal_to(1), 'Amount of processed files is wrong')
		assert_that(summary.total_failures(), equal_to(1), 'Amount of failed files is wrong')
		assert_that(summary.total_successes(), equal_to(0), 'Amount of successed files is wrong')

if __name__ == '__main__':
	unittest.main()    



class MediaFileTracking():
	def __init__(self, descriptor):
		self.__descriptor=descriptor
		self.__events_container=TrackingEventsProcessor([])

	@staticmethod
	def create_from(descriptor):
		return MediaFileTracking(descriptor)

	@property
	def mediafile(self):
		return self.__descriptor

	@property
	def tracking_events(self):
		return self.__events_container.get_events_registry()

	def track_activity(self, track_event):
		self.__descriptor = self.__events_container.process(self.__descriptor, track_event)

class MediaFileTrackingEvent():

	def __init__(self, entry_type, description, context_information={}):
		self.__type=entry_type
		self.__description=description
		self.__context_info=context_information

	def type(self):
		return self.__type

	def description(self):
		return self.__description

	def context_information(self):
		return self.__context_info

	def process(self, file_descriptor):
		return file_descriptor

class MediaFileDescriptor:
	def __init__(self):
		self.__sourcepath=None
		self.__destinationpath=None

	@property
	def sourcepath(self):
		return self.__sourcepath

	@sourcepath.setter
	def sourcepath(self, value):
		self.__sourcepath=value

	@property
	def destinationpath(self):
		return self.__destinationpath

	@destinationpath.setter
	def destinationpath(self, value):
		self.__destinationpath=value

class MediaFileDescriptorBuilder:
	def __init__(self):
		self.__sourcepath=None

	def sourcepath(self, sourcepath):
		self.__sourcepath=sourcepath
		return self

	def build(self):
		descriptor=MediaFileDescriptor()
		descriptor.sourcepath=self.__sourcepath

		return descriptor

class TrackingEventsBrowser:
	def __init__(self, entries):
		self.__tracking_groups={}
		self.__tracking_entries=entries

		self.__group_entries(entries)

	def __group_entries(self, entries):
		for entry in entries:
			self.__assign_event_to_group(entry)

	def __assign_event_to_group(self, tracking_entry):

		if self.__tracking_groups.has_key(tracking_entry.type()):
			self.__tracking_groups[tracking_entry.type()].append(tracking_entry)
		else:
			self.__tracking_groups[tracking_entry.type()]=[tracking_entry]			

	def size(self):
		return len(self.__tracking_entries)

	def is_empty(self):
		return len(self.__tracking_entries) == 0

	def get_all(self):
		return TrackingEventsBrowser(self.__tracking_entries)

	def get(self, index):
		return self.__tracking_entries[index]

	def of_type(self, event_type):
		if self.__tracking_groups.has_key(event_type):
			return TrackingEventsBrowser(self.__tracking_groups[event_type])
		else:
			return TrackingEventsBrowser([])

	def __iter__(self):
		return iter(self.__tracking_entries)

class TrackingEventsProcessor():
	def __init__(self, entries):
		self.__tracking_entries=entries

	def process(self, current_descriptor, tracking_entry):
		self.__tracking_entries.append(tracking_entry)
		return tracking_entry.process(current_descriptor)

	def get_events_registry(self):
		return TrackingEventsBrowser(self.__tracking_entries)



#https://github.com/hamcrest/PyHamcrest
#http://packages.python.org/PyHamcrest/

import gevent
import unittest
import os

from hamcrest import *
from longtang.actors.supervisors.id3taghandler import id3taghandler, messages, factory, configuration
from longtang.system import system
from longtang.actors import testutils
from longtang.common import domain as common_domain
from longtang.actors.tracking import tracking, facade

class TestId3TagHandlerSupervisor(unittest.TestCase):

	def test_creation(self):

		actor_system = system.ActorSystem()

		config = configuration.Id3TagHandlerConfigurationBuilder().override_tags(False)\
																  .offline_mode(False)\
																  .build()

		filepackaging_actor = actor_system.with_factory(factory.Id3TagSupervisorFactory(config),'id3tag-supervisor')

		try:
			assert_that(actor_system.find_by_id('id3tag-supervisor'), is_not(None), 'Id3 tag supervisor does not exist within system')
			assert_that(actor_system.find_by_id('id3tag-reader-actor'), is_not(None), 'Id3 reader actor does not exist within system')
			assert_that(actor_system.find_by_id('musicbrainz-actor'), is_not(None), 'Musicbrainz actor does not exist within system')

			assert_that(actor_system.find_by_id('id3tag-reader-actor').parent(), is_(equal_to(actor_system.find_by_id('id3tag-supervisor'))),'Parent actor is not correctly assigned')
			assert_that(actor_system.find_by_id('musicbrainz-actor').parent(), is_(equal_to(actor_system.find_by_id('id3tag-supervisor'))),'Parent actor is not correctly assigned')
			
		except exceptions.ActorNotFound as e:
			self.fail(str(e))

		actor_system.shutdown()

	def test_full_metadata_from_file(self):

		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')
		source_file = os.path.join(data_dir, 'with_id3taginfo.mp3')

		config = configuration.Id3TagHandlerConfigurationBuilder().override_tags(False)\
																  .offline_mode(False)\
																  .build()

		id3taghandler_actor = testutils.TestActorBuilder().withFactory(factory.Id3TagSupervisorFactory(config))\
															.withId('test-id3taghandler')\
															.terminationMoratorium(2)\
															.terminate_system(False)\
															.terminationType(messages.FileMetadataAvailable).build()

		tracking_facade = facade.MediaTrackingFacade.create_within(id3taghandler_actor.system())
		tracking_id = tracking_facade.create_tracking(source_file)

		id3taghandler_actor.tell(messages.InspectFileMetadata(source_file, tracking_id))

		assert_that(id3taghandler_actor.inspector().num_instances(messages.FileMetadataAvailable), is_(equal_to(1)), 'Total amount of FileMetadataAvailable messages received is wrong')

		message = id3taghandler_actor.inspector().get_first(messages.FileMetadataAvailable)

		assert_that(message.metadata().artist, is_(not_none()), 'Artist information is empty')
		assert_that(message.metadata().artist, is_(equal_to('Bon Jovi')), 'Artist information content is not as expected')

		assert_that(message.metadata().album, is_(not_none()), 'Album information is empty')
		assert_that(message.metadata().album, is_(equal_to('New Jersey')), 'Album information content is not as expected')

		assert_that(message.metadata().title, is_(not_none()), 'Title information is empty')
		assert_that(message.metadata().title, is_(equal_to('Ride Cowboy Ride')), 'Title information content is not as expected')

		assert_that(message.metadata().track_number, is_(not_none()), 'Track number information is empty')
		assert_that(message.metadata().track_number, is_(equal_to(8)), 'Track number information is not as expected')

		#Tracking information
		assert_that(message.tracking(), is_(equal_to(tracking_id)), 'Tracking id does not match')

		tracking_info = tracking_facade.lookup(tracking_id)

		assert_that(tracking_info.tracking_events.of_type('METADATA_AVAILABLE').is_empty(), is_(False),'Tracking events of type METADATA_AVAILABLE is empty')

	def test_partial_metadata_from_file(self):

		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')
		source_file = os.path.join(data_dir, 'partial_id3taginfo.mp3')

		config = configuration.Id3TagHandlerConfigurationBuilder().override_tags(False)\
																  .offline_mode(False)\
																  .build()

		id3taghandler_actor = testutils.TestActorBuilder().withFactory(factory.Id3TagSupervisorFactory(config))\
															.withId('test-id3taghandler')\
															.terminationMoratorium(20)\
															.terminate_system(False)\
															.terminationType(messages.FileMetadataAvailable).build()

		tracking_facade = facade.MediaTrackingFacade.create_within(id3taghandler_actor.system())
		tracking_id = tracking_facade.create_tracking(source_file)

		id3taghandler_actor.tell(messages.InspectFileMetadata(source_file, tracking_id))

		assert_that(id3taghandler_actor.inspector().num_instances(messages.FileMetadataAvailable), is_(equal_to(1)), 'Total amount of FileMetadataAvailable messages received is wrong')

		message = id3taghandler_actor.inspector().get_first(messages.FileMetadataAvailable)

		assert_that(message.metadata().artist, is_(not_none()), 'Artist information is empty')
		assert_that(message.metadata().artist, is_(equal_to('Bon Jovi')), 'Artist information content is not as expected')

		assert_that(message.metadata().album, is_(not_none()), 'Album information is empty')
		assert_that(message.metadata().album, is_(equal_to('New Jersey')), 'Album information content is not as expected')

		assert_that(message.metadata().title, is_(not_none()), 'Title information is empty')
		assert_that(message.metadata().title, is_(equal_to('Ride Cowboy Ride')), 'Title information content is not as expected')

		assert_that(message.metadata().track_number, is_(not_none()), 'Track number information is empty')
		assert_that(message.metadata().track_number, is_(equal_to(8)), 'Track number information is not as expected')

		assert_that(message.tracking(), is_(equal_to(tracking_id)), 'Tracking id does not match')

		tracking_info = tracking_facade.lookup(tracking_id)

		assert_that(tracking_info.tracking_events.of_type('METADATA_AVAILABLE').is_empty(), is_(False),'Tracking events of type METADATA_AVAILABLE is empty')		

	def test_empty_metadata_from_file(self):

		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')
		source_file = os.path.join(data_dir, 'emptytaginfo.mp3')

		config = configuration.Id3TagHandlerConfigurationBuilder().override_tags(False)\
																  .offline_mode(False)\
																  .build()

		id3taghandler_actor = testutils.TestActorBuilder().withFactory(factory.Id3TagSupervisorFactory(config))\
															.withId('test-id3taghandler')\
															.terminationMoratorium(20)\
															.terminate_system(False)\
															.terminationType(messages.FileMetadataNotFullyAvailable).build()

		tracking_facade = facade.MediaTrackingFacade.create_within(id3taghandler_actor.system())
		tracking_id = tracking_facade.create_tracking(source_file)

		id3taghandler_actor.tell(messages.InspectFileMetadata(source_file, tracking_id))

		assert_that(id3taghandler_actor.inspector().num_instances(messages.FileMetadataNotFullyAvailable), is_(equal_to(1)), 'Total amount of FileMetadataNotFullyAvailable messages received is wrong')

		message = id3taghandler_actor.inspector().get_first(messages.FileMetadataNotFullyAvailable)

		assert_that(message.source_file(), is_(not_none()), 'Source file information is empty')
		assert_that(message.tracking(), is_(equal_to(tracking_id)), 'Tracking id does not match')

		tracking_info = tracking_facade.lookup(tracking_id)

		assert_that(tracking_info.tracking_events.of_type('METADATA_NOT_AVAILABLE_FAILURE').is_empty(), is_(False),'Tracking events of type METADATA_NOT_AVAILABLE is empty')		

	def test_non_resolvable_metadata_from_file(self):

		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')
		source_file = os.path.join(data_dir, 'nonresolvable_id3taginfo.mp3')

		config = configuration.Id3TagHandlerConfigurationBuilder().override_tags(False)\
																  .offline_mode(False)\
																  .build()

		id3taghandler_actor = testutils.TestActorBuilder().withFactory(factory.Id3TagSupervisorFactory(config))\
															.withId('test-id3taghandler')\
															.terminationMoratorium(20)\
															.terminate_system(False)\
															.terminationType(messages.FileMetadataNotFullyAvailable).build()

		tracking_facade = facade.MediaTrackingFacade.create_within(id3taghandler_actor.system())
		tracking_id = tracking_facade.create_tracking(source_file)

		id3taghandler_actor.tell(messages.InspectFileMetadata(source_file, tracking_id))

		assert_that(id3taghandler_actor.inspector().num_instances(messages.FileMetadataNotFullyAvailable), is_(equal_to(1)), 'Total amount of FileMetadataNotFullyAvailable messages received is wrong')

		message = id3taghandler_actor.inspector().get_first(messages.FileMetadataNotFullyAvailable)

		assert_that(message.source_file(), is_(not_none()), 'Source file information is empty')
		assert_that(message.tracking(), is_(equal_to(tracking_id)), 'Tracking id does not match')

		tracking_info = tracking_facade.lookup(tracking_id)

		assert_that(tracking_info.tracking_events.of_type('METADATA_NOT_AVAILABLE_FAILURE').is_empty(), is_(False),'Tracking events of type METADATA_NOT_AVAILABLE is empty')

	def test_wrong_filepath(self):

		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')
		source_file = os.path.join(data_dir, 'mpty_id3taginfo.mp3')

		config = configuration.Id3TagHandlerConfigurationBuilder().override_tags(False)\
																  .offline_mode(False)\
																  .build()

		id3taghandler_actor = testutils.TestActorBuilder().withFactory(factory.Id3TagSupervisorFactory(config))\
															.withId('test-id3taghandler')\
															.terminationMoratorium(20)\
															.terminate_system(False)\
															.terminationType(messages.FileMetadataCouldNotBeEvaluated).build()

		tracking_facade = facade.MediaTrackingFacade.create_within(id3taghandler_actor.system())
		tracking_id = tracking_facade.create_tracking(source_file)

		id3taghandler_actor.tell(messages.InspectFileMetadata(source_file, tracking_id))

		assert_that(id3taghandler_actor.inspector().num_instances(messages.FileMetadataCouldNotBeEvaluated), is_(equal_to(1)), 'Total amount of FileMetadataCouldNotBeEvaluated messages received is wrong')

		message = id3taghandler_actor.inspector().get_first(messages.FileMetadataCouldNotBeEvaluated)

		assert_that(message.source_file(), is_(not_none()), 'Source file information is empty')
		assert_that(message.tracking(), is_(equal_to(tracking_id)), 'Tracking id does not match')

		tracking_info = tracking_facade.lookup(tracking_id)

		assert_that(tracking_info.tracking_events.of_type('METADATA_EVALUATION_FAILURE').is_empty(), is_(False),'Tracking events of type METADATA_EVALUATION_FAILED is empty')

	def test_override_metadata_from_file(self):

		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')
		source_file = os.path.join(data_dir, '09-override_id3taginfo.mp3')

		config = configuration.Id3TagHandlerConfigurationBuilder().override_tags(True)\
																  .offline_mode(False)\
																  .build()

		id3taghandler_actor = testutils.TestActorBuilder().withFactory(factory.Id3TagSupervisorFactory(config))\
															.withId('test-id3taghandler')\
															.terminationMoratorium(20)\
															.terminate_system(False)\
															.terminationType(messages.FileMetadataAvailable).build()

		tracking_facade = facade.MediaTrackingFacade.create_within(id3taghandler_actor.system())
		tracking_id = tracking_facade.create_tracking(source_file)

		id3taghandler_actor.tell(messages.InspectFileMetadata(source_file, tracking_id))

		assert_that(id3taghandler_actor.inspector().num_instances(messages.FileMetadataAvailable), is_(equal_to(1)), 'Total amount of FileMetadataAvailable messages received is wrong')

		message = id3taghandler_actor.inspector().get_first(messages.FileMetadataAvailable)

		assert_that(message.metadata().artist, is_(not_none()), 'Artist information is empty')
		assert_that(message.metadata().artist, is_(equal_to('Bon Jovi')), 'Artist information content is not as expected')

		assert_that(message.metadata().album, is_(not_none()), 'Album information is empty')
		assert_that(message.metadata().album, is_(equal_to('New Jersey')), 'Album information content is not as expected')

		assert_that(message.metadata().title, is_(not_none()), 'Title information is empty')
		assert_that(message.metadata().title, is_(equal_to('Ride Cowboy Ride')), 'Title information content is not as expected')

		assert_that(message.metadata().track_number, is_(not_none()), 'Track number information is empty')
		assert_that(message.metadata().track_number, is_(equal_to(9)), 'Track number information is not as expected')

		assert_that(message.tracking(), is_(equal_to(tracking_id)), 'Tracking id does not match')

		tracking_info = tracking_facade.lookup(tracking_id)

		assert_that(tracking_info.tracking_events.of_type('METADATA_AVAILABLE').is_empty(), is_(False),'Tracking events of type METADATA_AVAILABLE is empty')

	def test_offline_mode(self):

		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')
		source_file = os.path.join(data_dir, '09-offline_mode.mp3')

		config = configuration.Id3TagHandlerConfigurationBuilder().override_tags(False)\
																  .offline_mode(True)\
																  .build()

		id3taghandler_actor = testutils.TestActorBuilder().withFactory(factory.Id3TagSupervisorFactory(config))\
															.withId('test-id3taghandler')\
															.terminationMoratorium(20)\
															.terminate_system(False)\
															.terminationType(messages.FileMetadataNotFullyAvailable).build()

		tracking_facade = facade.MediaTrackingFacade.create_within(id3taghandler_actor.system())
		tracking_id = tracking_facade.create_tracking(source_file)

		id3taghandler_actor.tell(messages.InspectFileMetadata(source_file, tracking_id))

		assert_that(id3taghandler_actor.inspector().num_instances(messages.FileMetadataNotFullyAvailable), is_(equal_to(1)), 'Total amount of FileMetadataNotFullyAvailable messages received is wrong')

		message = id3taghandler_actor.inspector().get_first(messages.FileMetadataNotFullyAvailable)

		assert_that(message.tracking(), is_(equal_to(tracking_id)), 'Tracking id does not match')

		tracking_info = tracking_facade.lookup(tracking_id)

		assert_that(tracking_info.tracking_events.of_type('METADATA_NOT_AVAILABLE_FAILURE').is_empty(), is_(False),'Tracking events of type METADATA_NOT_AVAILABLE_FAILURE is empty')		



import unittest
import os

from longtang.actors import testutils
from longtang.actors.media.id3tag import id3tagchecker, messages
from hamcrest import *


class TestID3TagCheckerActor(unittest.TestCase):

	def test_with_id3taginfo(self):
		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')

		id3tag_tester = testutils.TestActorBuilder().withType(id3tagchecker.ID3TagCheckerActor).build()

		source_file = os.path.join(data_dir, 'withtaginfo.mp3')

		id3tag_tester.tell(messages.CheckFileMetadata(source_file,'tracking-id'))

		assert_that(id3tag_tester.inspector().num_instances(messages.FileMetadataIsComplete), is_(equal_to(1)), 'Total amount of FileMetadataIsComplete messages received is wrong')

		message = id3tag_tester.inspector().get_first(messages.FileMetadataIsComplete)

		assert_that(message.source(), is_(equal_to(source_file)), 'Source file is not present in response')
		assert_that(message.metadata().artist, is_(equal_to('Bon Jovi')), 'Artist is not present within metadata')
		assert_that(message.metadata().album, is_(equal_to('New Jersey')), 'Album is not present within metadata')
		assert_that(message.metadata().title, is_(equal_to('Ride Cowboy Ride')), 'Title is not present within metadata')
		assert_that(message.metadata().track_number, is_(equal_to(8)), 'Track is not present within metadata')
		assert_that(message.metadata().track_number, is_(equal_to(8)), 'Track is not present within metadata')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id not present within message')

	def test_without_id3taginfo(self):
		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')

		id3tag_tester = testutils.TestActorBuilder().withType(id3tagchecker.ID3TagCheckerActor).build()

		source_file = os.path.join(data_dir, 'withouttaginfo.mp3')

		id3tag_tester.tell(messages.CheckFileMetadata(source_file, 'tracking-id'))

		assert_that(id3tag_tester.inspector().num_instances(messages.FileMetadataIsIncomplete), is_(equal_to(1)), 'Total amount of FileMetadataIsIncomplete messages received is wrong')

		message = id3tag_tester.inspector().get_first(messages.FileMetadataIsIncomplete)

		assert_that(message.source(), is_(equal_to(source_file)), 'Source file is not present in response')
		assert_that(message.metadata().artist, is_(none()), 'Artist is not present within metadata')
		assert_that(message.metadata().album, is_(none()), 'Album is not present within metadata')
		assert_that(message.metadata().title, is_(none()), 'Title is not present within metadata')
		assert_that(message.metadata().track_number, is_(none()), 'Track is not present within metadata')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id not present within message')

	def test_empty_id3taginfo(self):
		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')

		id3tag_tester = testutils.TestActorBuilder().withType(id3tagchecker.ID3TagCheckerActor).build()

		source_file = os.path.join(data_dir, 'emptytaginfo.mp3')

		id3tag_tester.tell(messages.CheckFileMetadata(source_file, 'tracking-id'))

		assert_that(id3tag_tester.inspector().num_instances(messages.FileMetadataIsIncomplete), is_(equal_to(1)), 'Total amount of FileMetadataIsIncomplete messages received is wrong')

		message = id3tag_tester.inspector().get_first(messages.FileMetadataIsIncomplete)

		assert_that(message.source(), is_(equal_to(source_file)), 'Source file is not present in response')
		assert_that(message.metadata().artist, is_(none()), 'Artist is not present within metadata')
		assert_that(message.metadata().album, is_(none()), 'Album is not present within metadata')
		assert_that(message.metadata().title, is_(none()), 'Title is not present within metadata')
		assert_that(message.metadata().track_number, is_(none()), 'Track is not present within metadata')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id not present within message')

	def test_wrong_file_path(self):
		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')

		id3tag_tester = testutils.TestActorBuilder().withType(id3tagchecker.ID3TagCheckerActor).build()

		source_file = os.path.join(data_dir, '_id3taginfo.mp3')

		id3tag_tester.tell(messages.CheckFileMetadata(source_file, 'tracking-id'))

		assert_that(id3tag_tester.inspector().num_instances(messages.FileMetadataCouldNotBeChecked), is_(equal_to(1)), 'Total amount of FileMetadataCouldNotBeChecked messages received is wrong')

		message = id3tag_tester.inspector().get_first(messages.FileMetadataCouldNotBeChecked)

		assert_that(message.source_file(), is_(not_none()), 'Source file is not present in response')
		assert_that(message.source_file(), is_(equal_to(source_file)), 'Source file is not present in response')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id is not present in response')

	def test_missing_track_number_1digits(self):
		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')

		id3tag_tester = testutils.TestActorBuilder().withType(id3tagchecker.ID3TagCheckerActor).build()

		source_file = os.path.join(data_dir, '8_id3taginfo.mp3')

		id3tag_tester.tell(messages.CheckFileMetadata(source_file,'tracking-id'))

		assert_that(id3tag_tester.inspector().num_instances(messages.FileMetadataIsComplete), is_(equal_to(1)), 'Total amount of FileMetadataIsComplete messages received is wrong')

		message = id3tag_tester.inspector().get_first(messages.FileMetadataIsComplete)

		assert_that(message.source(), is_(equal_to(source_file)), 'Source file is not present in response')
		assert_that(message.metadata().artist, is_(equal_to('Bon Jovi')), 'Artist is not present within metadata')
		assert_that(message.metadata().album, is_(equal_to('New Jersey')), 'Album is not present within metadata')
		assert_that(message.metadata().title, is_(equal_to('Ride Cowboy Ride')), 'Title is not present within metadata')
		assert_that(message.metadata().track_number, is_(equal_to(8)), 'Track is not present within metadata')
		assert_that(message.metadata().track_number, is_(equal_to(8)), 'Track is not present within metadata')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id not present within message')

	def test_missing_track_number_2digits(self):
		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')

		id3tag_tester = testutils.TestActorBuilder().withType(id3tagchecker.ID3TagCheckerActor).build()

		source_file = os.path.join(data_dir, '08_id3taginfo.mp3')

		id3tag_tester.tell(messages.CheckFileMetadata(source_file,'tracking-id'))

		assert_that(id3tag_tester.inspector().num_instances(messages.FileMetadataIsComplete), is_(equal_to(1)), 'Total amount of FileMetadataIsComplete messages received is wrong')

		message = id3tag_tester.inspector().get_first(messages.FileMetadataIsComplete)

		assert_that(message.source(), is_(equal_to(source_file)), 'Source file is not present in response')
		assert_that(message.metadata().artist, is_(equal_to('Bon Jovi')), 'Artist is not present within metadata')
		assert_that(message.metadata().album, is_(equal_to('New Jersey')), 'Album is not present within metadata')
		assert_that(message.metadata().title, is_(equal_to('Ride Cowboy Ride')), 'Title is not present within metadata')
		assert_that(message.metadata().track_number, is_(equal_to(8)), 'Track is not present within metadata')
		assert_that(message.metadata().track_number, is_(equal_to(8)), 'Track is not present within metadata')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id not present within message')

	def test_missing_track_number_3digits(self):
		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')

		id3tag_tester = testutils.TestActorBuilder().withType(id3tagchecker.ID3TagCheckerActor).build()

		source_file = os.path.join(data_dir, '010_id3taginfo.mp3') 

		id3tag_tester.tell(messages.CheckFileMetadata(source_file,'tracking-id'))

		assert_that(id3tag_tester.inspector().num_instances(messages.FileMetadataIsComplete), is_(equal_to(1)), 'Total amount of FileMetadataIsComplete messages received is wrong')

		message = id3tag_tester.inspector().get_first(messages.FileMetadataIsComplete)

		assert_that(message.source(), is_(equal_to(source_file)), 'Source file is not present in response')
		assert_that(message.metadata().artist, is_(equal_to('Bon Jovi')), 'Artist is not present within metadata')
		assert_that(message.metadata().album, is_(equal_to('New Jersey')), 'Album is not present within metadata')
		assert_that(message.metadata().title, is_(equal_to('Ride Cowboy Ride')), 'Title is not present within metadata')
		assert_that(message.metadata().track_number, is_(equal_to(10)), 'Track is not present within metadata')
		assert_that(message.metadata().track_number, is_(equal_to(10)), 'Track is not present within metadata')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id not present within message')			


from longtang.common import mediatracking

class FilePackagingFinished(mediatracking.MediaFileTrackingEvent):
	def __init__(self, packaged_file):
		mediatracking.MediaFileTrackingEvent.__init__(self, "FILE_PACKAGING_DONE", "File has been successfully packaged")
		self.__packaged_file=packaged_file

	def process(self, file_descriptor):

		file_descriptor.destinationpath=self.__packaged_file
		return file_descriptor

class FileCopyFailed(mediatracking.MediaFileTrackingEvent):
	def __init__(self):
		mediatracking.MediaFileTrackingEvent.__init__(self, "FILE_COPY_FAILURE", "File could not be copied")

class FolderCreationFailed(mediatracking.MediaFileTrackingEvent):
	def __init__(self):
		mediatracking.MediaFileTrackingEvent.__init__(self, "FOLDER_CREATION_FAILURE", "Target folder could not be created")

class MetadataWritingFailed(mediatracking.MediaFileTrackingEvent):
	def __init__(self):
		mediatracking.MediaFileTrackingEvent.__init__(self, "METADATA_WRITING_FAILURE", "Metadata could not be persisted")

class PackagingStarted(mediatracking.MediaFileTrackingEvent):
	def __init__(self):
		mediatracking.MediaFileTrackingEvent.__init__(self, "PACKAGING_STARTED", "File packaging process started")
		
class TargetFolderCreated(mediatracking.MediaFileTrackingEvent):
	def __init__(self):
		mediatracking.MediaFileTrackingEvent.__init__(self, "TARGET_FOLDER_CREATED", "Target folder created")

class MediaFileCopied(mediatracking.MediaFileTrackingEvent):
	def __init__(self):
		mediatracking.MediaFileTrackingEvent.__init__(self, "MEDIAFILE_COPIED", "Media file copied")

class MetadataUpdated(mediatracking.MediaFileTrackingEvent):
	def __init__(self):
		mediatracking.MediaFileTrackingEvent.__init__(self, "METADATA_UPDATED", "Media file metadata updated")

class FileRenamingFailed(mediatracking.MediaFileTrackingEvent):
	def __init__(self):
		mediatracking.MediaFileTrackingEvent.__init__(self, "FILE_RENAMING_FAILURE", "Media file renaming process failed")




# -*- coding: utf-8 -*-

import messages
import domain
import os
import shutil

from longtang.actors import actors

class FileManagerActor(actors.Actor):

	def receive(self, message):

		if isinstance(message, messages.RenameFileFromMetadata):
			
			self.logger().info(u'File renaming using metadata requested .....{0}'.format(message.metadata()), message.tracking()) 

			target_dir = os.path.dirname(message.source_file())

			current_filename, current_ext = os.path.splitext(message.source_file())
			new_filename = u'{track:02d} - {title}{ext}'.format(track=message.metadata().track_number, title=message.metadata().title, ext=current_ext)
			new_filepath = os.path.join(target_dir, new_filename)

			self.logger().info(u'Resolved file name....\'{0}\''.format(new_filename), message.tracking())

			try:
				os.rename(message.source_file(), new_filepath)

				self.sender().tell(\
					messages.FileSuccessfullyRenamed(message.source_file(), message.metadata(), new_filepath, message.tracking()),\
					self.myself())

			except OSError as e:

				self.logger().error(u'File rename could not be done. Reason: {0}'.format(str(e)), message.tracking())

				self.sender().tell(\
					messages.RenameFileFromMetadataFailed(message.tracking(), message, str(e)), self.myself())
 

		elif isinstance(message, messages.CopyFileTo):

			self.logger().info(u'Copying file to {0}'.format(message.copy_to_path()), message.tracking())
			
			current_filepath, current_filename = os.path.split(message.source_file())
			copied_filepath = os.path.join(message.copy_to_path(), current_filename)

			try:
				shutil.copy2(message.source_file(),os.path.join(message.copy_to_path(), copied_filepath))

				self.sender().tell(\
					messages.FileSuccessfullyCopied(message.source_file(), message.metadata(), copied_filepath, message.tracking()),\
					self.myself())	

			except IOError as e:

				self.logger().error(u'File copy operation could not be done. Reason: {0}'.format(str(e)), message.tracking())

				self.sender().tell(\
					messages.CopyFileToFailed(message.tracking(), message, str(e)), self.myself())
		



import messages

from longtang.actors import actors
from longtang.actors.inbound import filepoller, messages as fp_messages
from longtang.actors.supervisors.filepackaging import filepackaging, messages as packaging_messages
from longtang.actors.supervisors.id3taghandler import id3taghandler, messages as id3tag_messages, factory, configuration as id3tag_config

class FlowConductorSupervisor(actors.Actor):

	def __init__(self, config, system_ref, unique_id):
		actors.Actor.__init__(self, system_ref, unique_id)

		self.__config=config

	def pre_setup(self):
		
		self.logger().info('Initializing actors...')

		self.logger().info('Initializing filepackaging-actor...')
		self.__filepackaging_ref = self.context().from_type(filepackaging.FilePackagingSupervisor,'filepackaging-actor')

		id3tagconfig = id3tag_config.Id3TagHandlerConfigurationBuilder().override_tags(self.__config.override_tags)\
																		  .offline_mode(self.__config.offline_mode)\
																		  .build()

		self.logger().info('Initializing id3taghandler-actor...')
		self.__id3taghandler_ref = self.context().with_factory(factory.Id3TagSupervisorFactory(id3tagconfig),'id3taghandler-actor')
	
	def receive(self, message):
		if isinstance(message, messages.MediaFileAvailable):
			self.context().get_by_id('id3taghandler-actor').tell(id3tag_messages.InspectFileMetadata(message.filepath(), message.tracking()), self)
		elif isinstance(message, id3tag_messages.FileMetadataAvailable):
			self.context().get_by_id('filepackaging-actor').tell(packaging_messages.PerformFilePackaging(message.source_file(), message.metadata(), self.__config.generate_at, message.tracking()), self)
		elif isinstance(message, packaging_messages.FilePackagingFinished):
			self.parent().tell(messages.MediaFileHasBeenProcessed(message.tracking()), self.myself())
		elif isinstance(message, id3tag_messages.FileMetadataCouldNotBeEvaluated):
			self.logger().error('Mediafile metadata evaluation failed...', message.source_file())
			self.parent().tell(messages.MediaFileProcessingFailed(message.tracking(), message), self.myself())
		elif isinstance(message, packaging_messages.FilePackagingFailure):
			self.logger().error('Mediafile metadata evaluation failed...', message.source_file())
			self.parent().tell(messages.MediaFileProcessingFailed(message.tracking(), message), self.myself())
		elif isinstance(message, id3tag_messages.FileMetadataNotFullyAvailable):
			self.logger().error('Mediafile metadata evaluation failed...', message.source_file())
			self.parent().tell(messages.MediaFileProcessingFailed(message.tracking(), message), self.myself())			
		else:
			self.notify_marooned_message(message)		


import messages
import domain
import medianame

from longtang.actors import actors

class ID3TagCheckerActor(actors.Actor):
	def receive(self, message):
		if isinstance(message, messages.CheckFileMetadata):

			self.logger().info(u'Checking ID3Tag information from file ...', message.tracking())

			try:
				reader = domain.ID3TagReaderFactory.createFromType(message.source())

				metadata = reader.extract_metadata()

				self.logger().info(u'Metadata information extracted: {0}'.format(metadata), message.tracking())

				if not metadata.missing_tags():
					self.sender().tell(messages.FileMetadataIsComplete(message.source(), metadata, message.tracking()), self.myself())
				else:

					if metadata.is_track_number_missing():
						evaluator = medianame.MedianameEvaluator.create_from_path(message.source())
						evaluation = evaluator.evaluate()

						if evaluation.track_number is not None:
							metadata.track_number = evaluation.track_number

					if not metadata.missing_tags():

						self.logger().info(u'Missing metadata guessed from name: {0}'.format(metadata), message.tracking())

						self.sender().tell(messages.FileMetadataIsComplete(message.source(), metadata, message.tracking()), self.myself())
					else:
						self.sender().tell(messages.FileMetadataIsIncomplete(message.source(), metadata, message.tracking()), self.myself())
			except IOError as e:
					self.logger().error(u'Metadata could not be checked. Reason: {0}'.format(str(e)), message.tracking())
					self.sender().tell(messages.FileMetadataCouldNotBeChecked(message.tracking(), message, str(e)), self.myself())
		else: 
			self.notify_marooned_message(message)


from longtang.actors import messages

class StartPolling(messages.ActorMessage):

	def __init__(self, source):
		messages.ActorMessage.__init__(self)
		self.__source=source

	def source(self):
		return self.__source

class FilePollingDone(messages.ActorMessage):
	def __init__(self):
		messages.ActorMessage.__init__(self)	

class AudioFileFound(messages.ActorMessage):

	def __init__(self, full_filepath):
		self.__full_filepath = full_filepath

	def filepath(self):
		return self.__full_filepath


#http://www.gevent.org/gevent.html#greenlet-objects

import gevent
import domain

from gevent.queue import Queue
from gevent.local import local
from longtang.actors import messages

class Actor(gevent.Greenlet):

    def __init__(self, system_ref, unique_id):
        gevent.Greenlet.__init__(self)
        self.__local_context=local()
        self.__inbox = Queue()
        self.__unique_id = unique_id
        self.__system_ref = system_ref
        self.__status = domain.ActorStatus.READY

    def status(self):
        return self.__status

    def receive(self, message):
        raise NotImplemented()

    def pre_setup(self):
        pass

    def _run(self):

        self.__status = domain.ActorStatus.RUNNING

        self.logger().debug('Running {0}'.format(self.__unique_id))

        self.running = True

        while self.running:
            envelope = self.__inbox.get()
            self.__local_context.sender=envelope.sender()
            self.__on_message(envelope.message())

    def __on_message(self,message):

        # 1- If current message is PoisonPill, then begin termination
        # 2- If current message is Terminated. If all children are dead then, current actor should call call self._silently_end()


        #TODO Maybe we could use a switch in here instead of an if statement
        if not self.is_terminating():
            
            self.logger().debug('New message received...[actor_id: {0}, message: {1}]'.format(self.unique_id(), message.__class__.__name__))
            
            if isinstance(message, messages.PoisonPill):
               
                # 1- If current has children, then should send each of them a poison pill and set current as TERMINATING
                # 2- If current has no children, call self._silently_end()

                self.__status = domain.ActorStatus.TERMINATING

                self.logger().debug('Start Termination...[actor_id: {0}, message: {1}]'.format(self.unique_id(), message.__class__.__name__))

                if self.__system_ref.have_children(self.unique_id()):

                    self.logger().debug('Notifying children...[actor_id: {0}, message: {1}]'.format(self.unique_id(), message.__class__.__name__))

                    children = self.__system_ref.get_children(self.unique_id())

                    for child in children:
                        if not child.is_terminated():
                            child.tell(messages.PoisonPill(), self.myself())
                else:
                    self.logger().debug('No children..so we kill ourself...[actor_id: {0}, message: {1}]'.format(self.unique_id(), message.__class__.__name__))
                    self._silently_end()
            else:
                self.receive(message)
        else:
            if isinstance(message, messages.Terminated):

                self.logger().debug('Child actor {2} finished...[actor_id: {0}, message: {1}]'.format(self.unique_id(), message.__class__.__name__, message.actor_id()))

                children = self.__system_ref.get_children(self.unique_id())

                should_terminate=True

                for child in children:
                    if not child.is_terminated():
                        should_terminate = False
                        break

                if should_terminate:
                    self.logger().debug('All children dead...terminating ourself...[actor_id: {0}, message: {1}]'.format(self.unique_id(), message.__class__.__name__))
                    self._silently_end()
            else:
                if self.__system_ref.is_child_of(self.sender().unique_id(), self.unique_id()):
                    self.receive(message)
                else:
                    self.logger().debug('Message ignored since we\'re in termination state. [Message: {0}]'.format(message))


    def context(self):
        return domain.SystemRefWrapper(self.__system_ref, self)

    def logger(self):
        return domain.LoggerHelper(self.__system_ref, self)

    def sender(self):
        return self.__local_context.sender

    def parent(self):
        return self.__system_ref.get_parent(self.__unique_id)

    def myself(self):
        return gevent.getcurrent()        

    def unique_id(self):
        return self.__unique_id

    def is_terminating(self):
        return self.__status is domain.ActorStatus.TERMINATING

    def is_terminated(self):
        return self.__status is domain.ActorStatus.TERMINATED

    def notify_marooned_message(self, message):
        self.__system_ref.notify_marooned_message(message, self.unique_id())

    def _silently_end(self):

        self.__status = domain.ActorStatus.TERMINATED

        self.logger().debug('Silently ending...[actor_id: {0}, sender: {1}]'.format(self.unique_id(), self.__local_context.sender))

        #TODO: Ufff...i need to check this one
        if self.parent().unique_id() != 'dead-letters':
            self.parent().tell(messages.Terminated(self.__unique_id))
            
        raise gevent.GreenletExit()

    def tell(self,message, sender=None):

        self.logger().debug('Telling....{0}_{1}'.format(self.__unique_id, message))

        self.__inbox.put(domain.Envelope(message, sender))

    def __str__(self):
        return "Actor[type={0}, unique_id={1}]".format(self.__class__.__name__, self.__unique_id)


# -*- coding: utf-8 -*-

#http://docs.python.org/2/library/unittest.html#unittest.TestCase.assertEqual

import unittest
import os

from longtang.actors.inbound import filepoller, messages as fp_messages
from longtang.actors import testutils, messages

class TestFilePollerActor(unittest.TestCase):

    def test_filepoller(self):

    	data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')

        poller_tester = testutils.TestActorBuilder().withType(filepoller.FilePoller).build()

        poller_tester.tell(fp_messages.StartPolling(data_dir))

        self.assertEqual(poller_tester.inspector().num_instances(), 5, 'Total amount of messages received is wrong')
        self.assertEqual(poller_tester.inspector().num_instances(fp_messages.AudioFileFound), 3, 'Total amount of AudioFileFound messages received is wrong')
        self.assertEqual(poller_tester.inspector().num_instances(fp_messages.FilePollingDone), 1, 'Total amount of FilePollingDone messages received is wrong')
        self.assertEqual(poller_tester.inspector().num_instances(messages.Terminated), 1, 'Total amount of Terminated messages received is wrong')

        self.assertIsNotNone(poller_tester.inspector().get_first(fp_messages.AudioFileFound).filepath(), 'File descriptor is empty or wrong')

    def test_encoding_filepoller(self):

        data_dir = os.path.join(testutils.current_dirpath(__file__), 'anóther_daña')

        poller_tester = testutils.TestActorBuilder().withType(filepoller.FilePoller).build()

        poller_tester.tell(fp_messages.StartPolling(data_dir))

        self.assertEqual(poller_tester.inspector().num_instances(), 4, 'Total amount of messages received is wrong')
        self.assertEqual(poller_tester.inspector().num_instances(fp_messages.AudioFileFound), 2, 'Total amount of AudioFileFound messages received is wrong')
        self.assertEqual(poller_tester.inspector().num_instances(fp_messages.FilePollingDone), 1, 'Total amount of FilePollingDone messages received is wrong')
        self.assertEqual(poller_tester.inspector().num_instances(messages.Terminated), 1, 'Total amount of Terminated messages received is wrong')

        self.assertIsNotNone(poller_tester.inspector().get_first(fp_messages.AudioFileFound).filepath(), 'File descriptor is empty or wrong')

if __name__ == '__main__':
    unittest.main()    


from longtang.actors import domain

class SystemRef():
	def __init__(self, system_instance):
		self.__system_instance = system_instance

	def get_parent(self, unique_id):
		return self.__system_instance.get_parent(unique_id)

	def get_by_id(self,unique_id):
		return self.__system_instance.find_by_id(unique_id)

	def from_type(self, actor_type, unique_id, parent):
		return self.__system_instance.from_type(actor_type, unique_id, parent)

	def with_factory(self, factory_instance, unique_id, parent):
		return self.__system_instance.with_factory(factory_instance, unique_id, parent)		

	def get_child(self, child_unique_id, parent_unique_id):
		return self.__system_instance.get_child(child_unique_id, parent_unique_id)

	def get_children(self, parent_unique_id):
		return self.__system_instance.get_children(parent_unique_id)

	def have_children(self, parent_unique_id):
		return self.__system_instance.have_children(parent_unique_id)

	def notify_marooned_message(self, message, notifier):
		self.__system_instance.marooned_messages().report(notifier, message)

	def logger(self):
		return self.__system_instance.active_logger()

	def _system_instance(self):
		return self.__system_instance

class WritableSystemRef(SystemRef):
	def __init__(self, system_instance):
		SystemRef.__init__(self, system_instance)

	def register_actor(self, actor_ref):
		self._system_instance()._register_actor_ref(actor_ref)

	def update_actor(self, actor_ref):
		self._system_instance()._update_actor_ref(actor_ref)		

class ActorRef():
	def __init__(self, actor_instance, parent):
		self.__actor_instance = actor_instance
		self.__parent = parent

	def status(self):
		return self.__actor_instance.status()

	def context(self):
		return self.__actor_instance.context()

	def tell(self,message, sender=None):
		self.__actor_instance.tell(message, sender)

	def _instance(self):
		return self.__actor_instance

	def parent(self):
		return self.__parent

	def delegate_setup(self):
		self.__actor_instance.pre_setup()

	def kickoff(self):
		self.__actor_instance.start()

	def type(self):
		return self.__actor_instance.__class__

	def unique_id(self):
		return self.__actor_instance.unique_id()

	def is_terminated(self):
		return self.__actor_instance.is_terminated()

	def __str__(self):
		return "ActorRef[type={0}, unique_id={1}, parent={2}]".format(self.__actor_instance.__class__.__name__, self.__actor_instance.unique_id(), self.__parent)

class ActorFactory():
	def create(self, unique_id, system_ref):
		raise NotImplemented()


from longtang.actors import messages

class CreateTrackingEntry(messages.ActorMessage):
	def __init__(self, sourcepath):
		messages.ActorMessage.__init__(self)
		self.__sourcepath = sourcepath

	def sourcepath(self):
		return self.__sourcepath

class TrackingEntryCreated(messages.ActorMessage):
	def __init__(self, tracking_id):
		messages.ActorMessage.__init__(self)
		self.__tracking_id = tracking_id

	def tracking_id(self):
		return self.__tracking_id


class LookupTrackingEntry(messages.ActorMessage):
	def __init__(self, tracking_id):
		messages.ActorMessage.__init__(self)
		self.__tracking_id = tracking_id

	def tracking_id(self):
		return self.__tracking_id

class TrackingEntryFound(messages.ActorMessage):
	def __init__(self, tracking_info):
		messages.ActorMessage.__init__(self)
		self.__tracking_info = tracking_info

	def tracking_info(self):
		return self.__tracking_info	

class TrackingEntryNotFound(messages.ActorMessage):
	def __init__(self, tracking_id):
		messages.ActorMessage.__init__(self)
		self.__tracking_id = tracking_id

	def tracking_id(self):
		return self.__tracking_id

class RegisterTrackingEvent(messages.ActorMessage):
	def __init__(self, tracking_id, tracking_event):
		self.__tracking_id=tracking_id
		self.__tracking_event=tracking_event

	def tracking_id(self):
		return self.__tracking_id

	def tracking_event(self):
		return self.__tracking_event

class TrackingEventSuccessfullyRegistered(messages.ActorMessage):
	pass

class GenerateSummary(messages.ActorMessage):
	pass

class SummarySuccessfullyGenerated(messages.ActorMessage):
	def __init__(self, summary):
		self.__summary=summary

	def summary(self):
		return self.__summary


from longtang.actors.logger import messages

class Envelope:
    def __init__(self, message, sender):
        self.__message = message
        self.__sender = sender

    def message(self):
        return self.__message

    def sender(self):
        return self.__sender

class SystemRefWrapper():
    def __init__(self, system_ref, actor_inst):

        self.__system_ref = system_ref
        self.__actor_ref = actor_inst

    def from_type(self, actor_type, unique_id):
        self.__system_ref.from_type(actor_type, unique_id, self.__system_ref.get_by_id(self.__actor_ref.unique_id()))

    def with_factory(self, factory_instance, unique_id):
        self.__system_ref.with_factory(factory_instance, unique_id, self.__system_ref.get_by_id(self.__actor_ref.unique_id()))

    def child(self, child_unique_id):
        return self.__system_ref.get_child(child_unique_id, self.__actor_ref.unique_id())

    def get_by_id(self,unique_id):
        return self.__system_ref.get_by_id(unique_id)

    def _system_ref(self):
        return self.__system_ref

class LoggerHelper():

    def __init__(self, system_ref, actor_inst):
        self.__system_ref = system_ref
        self.__actor_inst = actor_inst

    def info(self, message, source_file=None):
        self.__system_ref.logger().info(self.__actor_inst.unique_id(), message, source_file)

    def error(self, message, source_file=None):
        self.__system_ref.logger().error(self.__actor_inst.unique_id(), message, source_file)

    def debug(self, message, source_file=None):
        self.__system_ref.logger().debug(self.__actor_inst.unique_id(), message, source_file)        

    def warn(self, message, source_file=None):
        self.__system_ref.logger().warn(self.__actor_inst.unique_id(), message, source_file)


class ActorStatus:
    READY = 'Ready'
    PREPARING = 'Preparing'
    RUNNING = 'Running'
    TERMINATING = 'Terminating'
    TERMINATED = 'Terminated'        


import messages

from longtang.actors import actors
from longtang.actors.tracking import tracking, facade
from longtang.actors.inbound import filepoller, messages as fp_messages
from longtang.actors.supervisors.flowconductor import flowconductor, factory, messages as fc_messages

class MainSupervisor(actors.Actor):

	def __init__(self, config, system_ref, unique_id):
		actors.Actor.__init__(self, system_ref, unique_id)

		self.__config=config
		self.__filecount = {}
		self.__caller = []

	def pre_setup(self):
		
		self.logger().info('Initializing actors...')
		self.logger().info('Initializing file-poller-actor...')

		self.__file_poller_ref = self.context().from_type(filepoller.FilePoller,'file-poller-actor')

		self.logger().info('Initializing flowconductor-actor...')

		self.__flowconductor_ref = self.context().with_factory(factory.FlowConductorSupervisorFactory(self.__config),'flowconductor-actor')

		self.logger().info('Initializing tracking-actor...')
		self.__tracking_facade = facade.MediaTrackingFacade.create_within(self.context())

	
	def receive(self, message):
		if isinstance(message, messages.KickOffMediaProcessing):
			self.__caller.append(self.sender())
			self.logger().info('Media processing has just started. [From: {0}, To: {1}]'.format(self.__config.read_from, self.__config.generate_at))
			self.context().get_by_id('file-poller-actor').tell(fp_messages.StartPolling(self.__config.read_from), self)

		elif isinstance(message, fp_messages.AudioFileFound):
			tracking_id = facade.MediaTrackingFacade.from_context(self.context()).create_tracking(message.filepath())
			self.__filecount[tracking_id] = None
			self.context().get_by_id('flowconductor-actor').tell(fc_messages.MediaFileAvailable(message.filepath(), tracking_id), self)
		
		elif isinstance(message, fc_messages.MediaFileHasBeenProcessed):
			self.terminate_if_done(message)

		elif isinstance(message, fc_messages.MediaFileProcessingFailed):
			self.terminate_if_done(message)

		else: 
			self.notify_marooned_message(message)		

	def terminate_if_done(self, message):

		self.__filecount.pop(message.tracking())

		if len(self.__filecount) == 0:
			self.logger().info('Terminating execution.....')
			summary = facade.MediaTrackingFacade.from_context(self.context()).retrieve_summary()
			self.__caller[0].tell(messages.MediaProcessingFinished(summary), self)



#https://github.com/hamcrest/PyHamcrest
#http://packages.python.org/PyHamcrest/

from longtang.common import mediatracking
import unittest

from hamcrest import *

class MediaTrackingTest(unittest.TestCase):

	def test_creation(self):

		full_filepath = '/dummy_path.mp3'
		descriptor = mediatracking.MediaFileDescriptorBuilder().sourcepath(full_filepath).build()
		file_tracking = mediatracking.MediaFileTracking.create_from(descriptor)

		assert_that(file_tracking.mediafile.sourcepath, is_(not_none()), 'Media file path is empty')
		assert_that(file_tracking.mediafile.sourcepath, is_(equal_to(full_filepath)), 'Media file path value is wrong')
		assert_that(file_tracking.tracking_events, is_(not_none()), 'Tracking events has not being initialized')
		assert_that(file_tracking.tracking_events.is_empty(), is_(True), 'Tracking events is not empty')

	def test_add_tracking_event(self):

		full_filepath = '/dummy_path.mp3'
		descriptor = mediatracking.MediaFileDescriptorBuilder().sourcepath(full_filepath).build()
		file_tracking = mediatracking.MediaFileTracking.create_from(descriptor)

		file_tracking.track_activity(mediatracking.MediaFileTrackingEvent('my-unique-type', 'short description', {'value-of-interest1':1000, 'value-of-interes2':'testing'}))		

		assert_that(file_tracking.tracking_events.is_empty(), is_(False), 'Tracking events is empty')
		assert_that(file_tracking.tracking_events.get_all(), is_(not_none()), 'All tracking event list is empty')
		assert_that(file_tracking.tracking_events.get_all().size(), is_(equal_to(1)), 'All tracking event list has not the right size')
		assert_that(file_tracking.tracking_events.of_type('my-unique-type').is_empty(), is_(False), 'Tracking events of type \'my-unique-type\' is empty')
		assert_that(file_tracking.tracking_events.of_type('my-unique-type').get(0), is_(not_none()), 'First tracking event could not be found')
		assert_that(file_tracking.tracking_events.of_type('my-unique-type').get_all(), is_(not_none()), 'First tracking event could not be found')
		assert_that(file_tracking.tracking_events.of_type('my-unique-type').get_all().size(), is_(equal_to(1)), 'Tracking event list of type \'my-unique-type\' has not the right size')

	def test_add_altering_tracking_event(self):
		pass


	



import unittest
import os

from longtang.actors import testutils
from longtang.actors.media.id3tag import id3tagwriter, messages, domain
from longtang.common import domain as common_domain
from hamcrest import *


class TestID3TagWriterActor(unittest.TestCase):

	def setUp(self):

		global source_file

		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')
		source_file = testutils.copy_to_tmp(source=os.path.join(data_dir, 'emptytaginfo.mp3'),suffix='.tmp.mp3')

	def tearDown(self):
		os.remove(source_file)

	def test_write_id3taginfo(self):

		id3tag_tester = testutils.TestActorBuilder().withType(id3tagwriter.ID3TagWriterActor).build()

		metadata_builder = common_domain.FileMetadataBuilder()
		metadata = metadata_builder.artist('My Test Artist').title('My Test Title').album('My Test Album').track_number('7').build()

		id3tag_tester.tell(messages.WriteFileMetadata(source_file, metadata,'tracking-id'))

		assert_that(id3tag_tester.inspector().num_instances(messages.FileMetadataUpdated), is_(equal_to(1)), 'Total amount of FileMetadataUpdated messages received is wrong')		

		message = id3tag_tester.inspector().get_first(messages.FileMetadataUpdated)

		assert_that(message.source(), is_(not_none()), 'Source file is empty')
		assert_that(message.source(), is_(equal_to(source_file)), 'Source file is not the expected value')
		assert_that(message.metadata(), is_(not_none()), 'Metadata information is empty')
		assert_that(message.metadata(), is_(instance_of(common_domain.FileMetadata)), 'Metadata information is not the right type')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id is missing')


	def test_written_file_id3taginfo(self):

		id3tag_tester = testutils.TestActorBuilder().withType(id3tagwriter.ID3TagWriterActor).build()

		metadata_builder = common_domain.FileMetadataBuilder()
		metadata = metadata_builder.artist('Artist#1').title('Title#1').album('Album#1').track_number('15').build()

		id3tag_tester.tell(messages.WriteFileMetadata(source_file, metadata, 'tracking-id'))

		assert_that(id3tag_tester.inspector().num_instances(messages.FileMetadataUpdated), is_(equal_to(1)), 'Total amount of FileMetadataUpdated messages received is wrong')		

		message = id3tag_tester.inspector().get_first(messages.FileMetadataUpdated)

		reader = domain.ID3TagReaderFactory.createFromType(message.source())

		assert_that(reader.artist(), is_(equal_to('Artist#1')), 'id3tag content for artist does not match')
		assert_that(reader.title(), is_(equal_to('Title#1')), 'id3tag content for title does not match')
		assert_that(reader.album(), is_(equal_to('Album#1')), 'id3tag content for album does not match')
		assert_that(reader.track_number(), is_(equal_to('15')), 'id3tag content for track number does not match')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id is missing')

	def test_write_id3taginfo_not_existing_file_failure(self):

		id3tag_tester = testutils.TestActorBuilder().withType(id3tagwriter.ID3TagWriterActor).build()

		metadata_builder = common_domain.FileMetadataBuilder()
		metadata = metadata_builder.artist('My Test Artist').title('My Test Title').album('My Test Album').track_number('7').build()

		id3tag_tester.tell(messages.WriteFileMetadata('dummy.mp3', metadata, 'tracking-id'))

		assert_that(id3tag_tester.inspector().num_instances(messages.WriteFileMetadataFailed), is_(equal_to(1)), 'Total amount of WriteFileMetadataFailed messages received is wrong')		

		message = id3tag_tester.inspector().get_first(messages.WriteFileMetadataFailed)

		assert_that(message.source(), is_(not_none()), 'Source file is empty')
		assert_that(message.metadata(), is_(not_none()), 'Metadata information is empty')
		assert_that(message.reason(), is_(not_none()), 'Reason information is empty')
		assert_that(message.source_message(), is_(not_none()), 'Source message information is empty')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id is missing')

	def test_write_id3taginfo_missing_metadata_failure(self):

		id3tag_tester = testutils.TestActorBuilder().withType(id3tagwriter.ID3TagWriterActor).build()

		metadata_builder = common_domain.FileMetadataBuilder()
		metadata = metadata_builder.artist('My Test Artist').title('My Test Title').album(None).track_number('7').build()

		id3tag_tester.tell(messages.WriteFileMetadata(source_file, metadata, 'tracking-id'))

		assert_that(id3tag_tester.inspector().num_instances(messages.WriteFileMetadataFailed), is_(equal_to(1)), 'Total amount of WriteFileMetadataFailed messages received is wrong')		

		message = id3tag_tester.inspector().get_first(messages.WriteFileMetadataFailed)

		assert_that(message.source(), is_(not_none()), 'Source file is empty')
		assert_that(message.metadata(), is_(not_none()), 'Metadata information is empty')
		assert_that(message.reason(), is_(not_none()), 'Reason information is empty')
		assert_that(message.source_message(), is_(not_none()), 'Source message information is empty')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id is missing')		



import messages
import events

from longtang.actors import actors
from longtang.actors.media.id3tag import id3tagchecker, messages as id3tag_messages
from longtang.actors.outbound.musicbrainz import musicbrainz, messages as mb_messages
from longtang.actors.tracking import facade

class Id3TagSupervisor(actors.Actor):

	def __init__(self, config, system_ref, unique_id):
		actors.Actor.__init__(self, system_ref, unique_id)

		self.__config=config

	def pre_setup(self):

		self.logger().info('Initializing actors...')

		self.logger().info('Creating id3tag-reader-actor ....... DONE')

		self.__id3tagchecker_ref = self.context().from_type(id3tagchecker.ID3TagCheckerActor,'id3tag-reader-actor')

		self.logger().info('Creating musicbrainz-actor ....... DONE')

		self.__musicbrainz_ref = self.context().from_type(musicbrainz.MusicMetadataActor,'musicbrainz-actor')

	def receive(self, message):

		if isinstance(message, messages.InspectFileMetadata):
			self.logger().info(u'Starting metadata inspection on file .....', message.tracking())
			facade.MediaTrackingFacade.from_context(self.context()).notify(message.tracking(), events.MetadataInspectionStarted())
			self.context().get_by_id('id3tag-reader-actor').tell(id3tag_messages.CheckFileMetadata(message.source_file(), message.tracking()), self.myself())

		elif isinstance(message, id3tag_messages.FileMetadataIsComplete):
			facade.MediaTrackingFacade.from_context(self.context()).notify(message.tracking(), events.MetadataAvailable())

			if self.__config.override_tags:
				self.context().get_by_id('musicbrainz-actor').tell(mb_messages.OverrideFileMetadata(message.source(), message.metadata(), message.tracking()), self.myself())				
			else:
				self.parent().tell(messages.FileMetadataAvailable(message.source(), message.metadata(), message.tracking()), self.myself())

		elif isinstance(message, id3tag_messages.FileMetadataIsIncomplete):

			if not self.__config.offline_mode:
				self.logger().info('There\'re some metadata fields missing. Trying with musicbrainz .....', message.tracking())
				facade.MediaTrackingFacade.from_context(self.context()).notify(message.tracking(), events.FileMetadataIncomplete())
				self.context().get_by_id('musicbrainz-actor').tell(mb_messages.FillUpMissingMetadataFields(message.source(), message.metadata(), message.tracking()), self.myself())
			else:
				self.logger().error('File metadata could not be fulfilled and offline mode is active.', message.tracking())
				facade.MediaTrackingFacade.from_context(self.context()).notify(message.tracking(), events.MetadataNotAvailable())
				self.parent().tell(messages.FileMetadataNotFullyAvailable(message.source(), message.tracking()), self.myself())				

		elif isinstance(message, mb_messages.OverrideFileMetadataDone):
			self.logger().info('File metadata overriden with MusicBrainz info .....', message.tracking())
			facade.MediaTrackingFacade.from_context(self.context()).notify(message.tracking(), events.MetadataAvailable())
			self.parent().tell(messages.FileMetadataAvailable(message.source(), message.metadata(), message.tracking()), self.myself())

		elif isinstance(message, mb_messages.FileMetadataSuccessfullyFilled):
			self.logger().info('File metadata found in MusicBrainz .....', message.tracking())
			facade.MediaTrackingFacade.from_context(self.context()).notify(message.tracking(), events.MetadataAvailable())
			self.parent().tell(messages.FileMetadataAvailable(message.source(), message.metadata(), message.tracking()), self.myself())

		elif isinstance(message, id3tag_messages.FileMetadataCouldNotBeChecked):
			self.logger().error('File metadata failed to be checked. Reason: {0}'.format(message.reason()), message.tracking())
			facade.MediaTrackingFacade.from_context(self.context()).notify(message.tracking(), events.MetadataEvaluationFailed())
			self.parent().tell(messages.FileMetadataCouldNotBeEvaluated(message.source_file(), message.tracking()), self.myself())

		elif isinstance(message, mb_messages.FillUpMissingMetadataFieldsFailed) or isinstance(message, mb_messages.OverrideFileMetadataFailed):
			self.logger().error('File metadata could not be fulfilled. Reason: {0}'.format(message.reason()), message.tracking())
			facade.MediaTrackingFacade.from_context(self.context()).notify(message.tracking(), events.MetadataNotAvailable())
			self.parent().tell(messages.FileMetadataNotFullyAvailable(message.source_file(), message.tracking()), self.myself())

		elif isinstance(message, mb_messages.MusicBrainzServiceFailed):
			self.logger().error('Musicbrainz service is not responding.', message.tracking())
			facade.MediaTrackingFacade.from_context(self.context()).notify(message.tracking(), events.MusicbrainzServiceFailed())
			self.parent().tell(messages.FileMetadataNotFullyAvailable(message.source(), message.tracking()), self.myself())			
		else: 
			self.notify_marooned_message(message)


#https://github.com/hamcrest/PyHamcrest
#http://packages.python.org/PyHamcrest/

import gevent
import unittest
from longtang.common import interactions

from hamcrest import *
from longtang.system import system
from longtang.actors import testutils, actors, messages

class TestInteractions(unittest.TestCase):

	def test_ask_method(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(InteractionTestActor,'interactions-test-actor')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, SimpleRequest())

		assert_that(response, is_(not_none()),'Response received is empty')
		assert_that(response, is_(instance_of(SimpleResponse)),'Response object type is not the expected one')

		actor_system.shutdown()


class InteractionTestActor(actors.Actor):

    def receive(self, message):
        gevent.sleep(1)
        self.sender().tell(SimpleResponse('This is my test response'), self.myself())

class SimpleRequest(messages.ActorMessage):
	pass

class SimpleResponse(messages.ActorMessage):
	def __init__(self, value):
		self.__value=value

	def value(self):
		return self.__value


import messages
import domain
import os

from longtang.actors import actors

class FolderManagerActor(actors.Actor):
	def receive(self, message):
		if isinstance(message, messages.CreateFolderFromMetadata):

			self.logger().info(u'Using metadata for folder creation....{0}'.format(message.metadata()), message.tracking())

			if message.metadata().missing_tags():
				self.logger().error(u'Folder could not be created from metadata {0}. Reason: One or more metadata fields are missing'.format(message.metadata()), message.tracking())
				self.sender().tell(messages.CreateFolderFromMetadataFailed(message.tracking(), message, 'One or more metadata fields are missing'), self.myself())
			else:
				try:
					
					full_dir_path = os.path.join(message.base_dir(), message.metadata().artist, message.metadata().album)

					if not os.path.exists(full_dir_path):
						os.makedirs(full_dir_path)

					self.sender().tell(\
						messages.FolderFromMetadataSuccessfullyCreated(message.source_file(), message.metadata(), full_dir_path, message.tracking()),\
						self.myself())

				except Exception as e:

					self.logger().error(u'Folder could not be created from metadata {0}. Reason: {1}'.format(message.metadata(), str(e)), message.tracking())
					self.sender().tell(messages.CreateFolderFromMetadataFailed(message.tracking(), message, str(e)), self.myself())


#http://docs.python.org/2/library/unittest.html#unittest.TestCase.assertEqual

import unittest
import os
import gevent

from hamcrest import *
from longtang.actors import testutils,actors,domain, messages
from longtang.system import system, domain as sys_domain

class TestSystemLogger(unittest.TestCase):

    def test_simple_event_message(self):

        sniffer = testutils.StdoutSniffer()

        actor_system = system.ActorSystem()
        logger_actor = actor_system.from_type(TestLoggerActor,'test-logger-actor')

        sniffer.start()

        logger_actor.tell(GenerateSimpleLogMessages('This is an INFO-LOG-TEST message', 'INFO'))
        logger_actor.tell(GenerateSimpleLogMessages('This is an ERROR-LOG-TEST message', 'ERROR'))
        logger_actor.tell(TerminateYourself())

        actor_system.wait_for(logger_actor)

        sniffer.stop()

        #TODO Logger output is not captured, as long as we implement the message sniffer, we should redefine this test
        #assert_that(sniffer.value(), contains_string('This is an INFO-LOG-TEST message'),'INFO message was not found')
        #assert_that(sniffer.value(), contains_string('This is an ERROR-LOG-TEST message'),'ERROR message was not found')

    def test_file_event_message(self):
        pass



class TestLoggerActor(actors.Actor):
    def receive(self, message):
        if isinstance(message, TerminateYourself):
            gevent.sleep(0.5)
            self.myself().tell(messages.PoisonPill())
        elif isinstance(message, GenerateSimpleLogMessages):
            if message.type() is 'INFO':
                self.logger().info(message.text())
            else:
                self.logger().error(message.text())
        else:        
            self.notify_marooned_message(message)   


class TerminateYourself(messages.ActorMessage):
    pass

class GenerateSimpleLogMessages(messages.ActorMessage):
    def __init__(self, message, type):
        self.__message=message
        self.__type=type
    def text(self):
        return self.__message
    def type(self):
        return self.__type

if __name__ == '__main__':
    unittest.main()


import domain
import itertools

from longtang.actors import actors, messages

class DeadLetters(actors.Actor):

	ROOT_PARENT='ROOT'

	def __init__(self, system_ref, unique_id, registry):
		actors.Actor.__init__(self,system_ref, unique_id)
		self.__registry=registry

	def receive(self, message):

		self.logger().info('Dead letter received....[reporter={0}, message={1}]'.format(message.reporter(), message.marooned_message()))

		self.__registry.append(message.reporter(),message.marooned_message())

class DeadLetterMessageReported(messages.ActorMessage):
	def __init__(self, reporter, message):
		self.__reporter=reporter
		self.__message=message

	def reporter(self):
		return self.__reporter

	def marooned_message(self):
		return self.__message

class DeadLettersManager:

	def __init__(self):
		self.__dead_letters_actor=None
		self.__registry=MaroonedMessagesRegistry()

	def report(self, reporter, message):
		self.__dead_letters_actor.tell(DeadLetterMessageReported(reporter,message))

	def reported_by(self, reporter):
		return self.__registry.reported_by(reporter)

	def register_within_system(self, system):
		self.__dead_letters_actor = system.with_factory(DeadLettersActorFactory(self.__registry), 'dead-letters', DeadLetters.ROOT_PARENT)
		return self.__dead_letters_actor

	def all(self):
		return self.__registry.all_messages()

class MaroonedMessagesRegistry:
	def __init__(self):
		self.__marooned_messages = {}

	def append(self, reporter, message):
		if reporter in self.__marooned_messages.keys():
			self.__marooned_messages[reporter].append(message)
		else:
			self.__marooned_messages[reporter] = [message]

	def reported_by(self, reporter):
		if reporter in self.__marooned_messages.keys():
			return MaroonedMessagesBrowser(self.__marooned_messages[reporter])
		else:
			return MaroonedMessagesBrowser()

	def all_messages(self):
		return MaroonedMessagesBrowser(list(itertools.chain(self.__marooned_messages.itervalues())))


class MaroonedMessagesBrowser():
	def __init__(self, messages=[]):
		self.__messages=messages

	def size(self):
		return len(self.__messages)

	def get(self, index):
		return self.__messages[index]

class DeadLettersActorFactory(domain.ActorFactory):

	def __init__(self, registry):
		self.__registry=registry

	def create(self, unique_id, system_ref):
		return DeadLetters(system_ref,unique_id,self.__registry)


from longtang.actors import messages

class CreateFolderFromMetadata(messages.TraceableActorMessage):
	def __init__(self, source_file, metadata, base_dir, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)

		self.__source_file = source_file
		self.__metadata = metadata
		self.__base_dir = base_dir

	def source_file(self):
		return self.__source_file

	def metadata(self):
		return self.__metadata

	def base_dir(self):
		return self.__base_dir

class FolderFromMetadataSuccessfullyCreated(messages.TraceableActorMessage):
	def __init__(self, source_file, metadata, new_dir_path, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)

		self.__source_file = source_file
		self.__metadata = metadata
		self.__new_dir_path = new_dir_path

	def source_file(self):
		return self.__source_file

	def metadata(self):
		return self.__metadata

	def new_dir_path(self):
		return self.__new_dir_path

class RenameFileFromMetadata(messages.TraceableActorMessage):
	def __init__(self, source_file, metadata, tracking):
		messages.TraceableActorMessage.__init__(self, tracking	)

		self.__source_file = source_file
		self.__metadata = metadata

	def source_file(self):
		return self.__source_file

	def metadata(self):
		return self.__metadata

class FileSuccessfullyRenamed(messages.TraceableActorMessage):
	def __init__(self, source_file, metadata, new_file, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)

		self.__source_file = source_file
		self.__metadata = metadata
		self.__new_file = new_file

	def source_file(self):
		return self.__source_file

	def metadata(self):
		return self.__metadata

	def new_file(self):
		return self.__new_file

class CopyFileTo(messages.TraceableActorMessage):
	def __init__(self, source_file, metadata, copy_to_path, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)

		self.__source_file = source_file
		self.__metadata = metadata
		self.__copy_to_path = copy_to_path

	def source_file(self):
		return self.__source_file

	def metadata(self):
		return self.__metadata

	def copy_to_path(self):
		return self.__copy_to_path

class FileSuccessfullyCopied(messages.TraceableActorMessage):
	def __init__(self, source_file, metadata, copied_file, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)

		self.__source_file = source_file
		self.__metadata = metadata
		self.__copied_file = copied_file

	def source_file(self):
		return self.__source_file

	def metadata(self):
		return self.__metadata

	def copied_file(self):
		return self.__copied_file

class CopyFileToFailed(messages.TraceableActorFailureMessage):
	def __init__(self, tracking, source_message, reason):
		messages.TraceableActorFailureMessage.__init__(self, tracking, reason, source_message)

	def source_file(self):
		return self.source_message().source_file()

	def metadata(self):
		return self.source_message().metadata()

	def copy_to_path(self):
		return self.source_message().copy_to_path()

class RenameFileFromMetadataFailed(messages.TraceableActorFailureMessage):
	def __init__(self, tracking,source_message, reason):
		messages.TraceableActorFailureMessage.__init__(self, tracking, reason, source_message)

	def source_file(self):
		return self.source_message().source_file()

	def metadata(self):
		return self.source_message().metadata()


class CreateFolderFromMetadataFailed(messages.TraceableActorFailureMessage):
	def __init__(self, tracking, source_message, reason):
		messages.TraceableActorFailureMessage.__init__(self, tracking, reason, source_message)

	def source_file(self):
		return self.source_message().source_file()

	def metadata(self):
		return self.source_message().metadata()

	def base_dir(self):
		return self.source_message().base_dir()


class ActorNotFound(Exception):
	def __init__(self, actor_id, msg=None):
		self.__actor_id = actor_id
		self.__msg = msg

	def __str__(self):
		return repr("Actor '{0}' could not be found within the system. {1}".format(self.__actor_id, self.__msg if self.__msg is not None else ''))


import messages
import hashlib
import summary
import sys

from longtang.actors import actors
from longtang.common import mediatracking

class TrackingActor(actors.Actor):

	def __init__(self, system_ref, unique_id):
		actors.Actor.__init__(self, system_ref, unique_id)

		self.__trackings={}

	def receive(self, message):
		if isinstance(message, messages.CreateTrackingEntry):
			self.logger().info(u'Creating tracking entry for source path {0} .....'.format(message.sourcepath()))
			descriptor = mediatracking.MediaFileDescriptorBuilder().sourcepath(message.sourcepath()).build()
			file_tracking = mediatracking.MediaFileTracking.create_from(descriptor)
			new_trackingid = hashlib.sha224(message.sourcepath().encode(sys.getdefaultencoding(),'replace')).hexdigest()

			self.__trackings[new_trackingid]=file_tracking

			self.sender().tell(messages.TrackingEntryCreated(new_trackingid), self.myself())

		elif isinstance(message, messages.LookupTrackingEntry):
			self.logger().info(u'Looking up tracking entry {0} .....'.format(message.tracking_id()))
			if self.__trackings.has_key(message.tracking_id()):
				self.sender().tell(messages.TrackingEntryFound(self.__trackings[message.tracking_id()]), self.myself())
			else:
				self.sender().tell(messages.TrackingEntryNotFound(message.tracking_id()), self.myself())	

		elif isinstance(message, messages.RegisterTrackingEvent):
			self.logger().info(u'Event of type \'{1}\' recieved for tracking id \'{0}\' .....'.format(message.tracking_id(),message.tracking_event().type()))

			if self.__trackings.has_key(message.tracking_id()):
				self.__trackings[message.tracking_id()].track_activity(message.tracking_event())

				if self.sender() is not None:
					self.sender().tell(messages.TrackingEventSuccessfullyRegistered(), self.myself())
			else:

				if self.sender() is not None:
					self.sender().tell(messages.TrackingEntryNotFound(message.tracking_id()), self.myself())

		elif isinstance(message, messages.GenerateSummary):
			self.sender().tell(messages.SummarySuccessfullyGenerated(summary.ActivitySummary.create(self.__trackings)), self.myself())			
		else: 
			self.notify_marooned_message(message)					


from longtang.actors import messages

class FileMetadataAvailable(messages.TraceableActorMessage):
	def __init__(self, source_file, metadata, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)

		self.__source_file = source_file
		self.__metadata = metadata

	def source_file(self):
		return self.__source_file

	def metadata(self):
		return self.__metadata

class FileMetadataNotFullyAvailable(messages.TraceableActorMessage):
	def __init__(self, source_file, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)

		self.__source_file = source_file

	def source_file(self):
		return self.__source_file

class InspectFileMetadata(messages.TraceableActorMessage):
	def __init__(self, source_file, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)

		self.__source_file = source_file

	def source_file(self):
		return self.__source_file

class FileMetadataCouldNotBeEvaluated(messages.TraceableActorMessage):
	def __init__(self, source_file, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)
		self.__source_file = source_file

	def source_file(self):
		return self.__source_file		


from longtang.actors import messages

class FileEventMessage(messages.ActorMessage):
	def __init__(self, actor, source_file, message):
		messages.ActorMessage.__init__(self)

		self.__message = message
		self.__source_file=source_file
		self.__actor=actor

	def message(self):
		return self.__message

	def actor(self):
		return self.__actor

	def source_file(self):
		return self.__source_file

	def type(self):
		raise NotImplemented()

class FileInformationEventMessage(FileEventMessage):
	def __init__(self, actor, source_file, message):
		FileEventMessage.__init__(self,actor, source_file, message)

	def type(self):
		return 'INFO'		

class FileErrorEventMessage(FileEventMessage):
	def __init__(self, actor, source_file, message):
		FileEventMessage.__init__(self,actor, source_file, message)	

	def type(self):
		return 'ERROR'

class FileDebugEventMessage(FileEventMessage):
	def __init__(self, actor, source_file, message):
		FileEventMessage.__init__(self,actor, source_file, message)	

	def type(self):
		return 'DEBUG'

class FileWarnEventMessage(FileEventMessage):
	def __init__(self, actor, source_file, message):
		FileEventMessage.__init__(self,actor, source_file, message)	

	def type(self):
		return 'WARN'					


import messages
import events

from longtang.actors import actors
from longtang.actors.filesystem import filemanager, foldermanager, messages as filesystem_messages
from longtang.actors.media.id3tag import id3tagwriter, messages as id3tag_messages
from longtang.actors.tracking import facade

class FilePackagingSupervisor(actors.Actor):

	def pre_setup(self):
		
		self.logger().info('Initializing actors...')

		self.__folder_manager_ref = self.context().from_type(foldermanager.FolderManagerActor,'id3-folder-manager-actor')

		self.logger().info('Creating id3-folder-manager-actor ....... DONE')

		self.__file_manager_ref = self.context().from_type(filemanager.FileManagerActor,'id3-file-manager-actor')

		self.logger().info('Creating id3-file-manager-actor ....... DONE')

		self.__file_tagger_ref = self.context().from_type(id3tagwriter.ID3TagWriterActor,'id3-file-tagger-actor')

		self.logger().info('Creating id3-file-tagger-actor ....... DONE')
	
	def receive(self, message):
		if isinstance(message, messages.PerformFilePackaging):
			
			self.logger().info('File to be packaged received', message.tracking())
			facade.MediaTrackingFacade.from_context(self.context()).notify(message.tracking(), events.PackagingStarted())
			self.context().get_by_id('id3-folder-manager-actor').tell(filesystem_messages.CreateFolderFromMetadata(message.source_file(),\
																													message.metadata(), \
																													message.target_dir(), message.tracking()), self)

		elif isinstance(message, filesystem_messages.FolderFromMetadataSuccessfullyCreated):

			self.logger().info('Target folder has been succesfully created', message.tracking())
			facade.MediaTrackingFacade.from_context(self.context()).notify(message.tracking(), events.TargetFolderCreated())
			self.context().get_by_id('id3-file-manager-actor').tell(filesystem_messages.CopyFileTo(message.source_file(),\
																									message.metadata(),\
																									message.new_dir_path(), message.tracking()), self)
		elif isinstance(message, filesystem_messages.FileSuccessfullyCopied):

			self.logger().info('Media file succesfully copied', message.tracking())
			facade.MediaTrackingFacade.from_context(self.context()).notify(message.tracking(), events.MediaFileCopied())
			self.context().get_by_id('id3-file-tagger-actor').tell(id3tag_messages.WriteFileMetadata(message.copied_file(),\
																										message.metadata(), message.tracking()), self)
		elif isinstance(message, id3tag_messages.FileMetadataUpdated):
			
			self.logger().info('Media file succesfully updated with metadata', message.tracking())
			facade.MediaTrackingFacade.from_context(self.context()).notify(message.tracking(), events.MetadataUpdated())
			self.context().get_by_id('id3-file-manager-actor').tell(filesystem_messages.RenameFileFromMetadata(message.source(),\
																												 message.metadata(), message.tracking()), self)
		elif isinstance(message, filesystem_messages.FileSuccessfullyRenamed):

			self.logger().info('Media file succesfully renamed according to metadata', message.tracking())
			facade.MediaTrackingFacade.from_context(self.context()).notify(message.tracking(), events.FilePackagingFinished(message.new_file()))
			self.parent().tell(messages.FilePackagingFinished(message.source_file(), message.tracking()), self)

		elif isinstance(message, filesystem_messages.CopyFileToFailed):

			self.logger().error('Media file could not be copied. Reason: {0}'.format(message.reason()), message.tracking())
			facade.MediaTrackingFacade.from_context(self.context()).notify(message.tracking(), events.FileCopyFailed())
			self.parent().tell(messages.FilePackagingFailure(message.source_file(), message.metadata(), message.reason(), message.tracking()), self)

		elif isinstance(message, filesystem_messages.RenameFileFromMetadataFailed):

			self.logger().error('Media file could not be renamed. Reason: {0}'.format(message.reason()), message.tracking())
			facade.MediaTrackingFacade.from_context(self.context()).notify(message.tracking(), events.FileRenamingFailed())
			self.parent().tell(messages.FilePackagingFailure(message.source_file(), message.metadata(), message.reason(), message.tracking()), self)

		elif isinstance(message, filesystem_messages.CreateFolderFromMetadataFailed):

			self.logger().error('Target folder could not be created. Reason: {0}'.format(message.reason()), message.tracking())
			facade.MediaTrackingFacade.from_context(self.context()).notify(message.tracking(), events.FolderCreationFailed())
			self.parent().tell(messages.FilePackagingFailure(message.source_file(), message.metadata(), message.reason(), message.tracking()), self)

		elif isinstance(message, id3tag_messages.WriteFileMetadataFailed):
			
			self.logger().error('Id3 tags could not be saved. Reason: {0}'.format(message.reason()), message.tracking())
			facade.MediaTrackingFacade.from_context(self.context()).notify(message.tracking(), events.MetadataWritingFailed())
			self.parent().tell(messages.FilePackagingFailure(message.source_file(), message.metadata(), message.reason(), message.tracking()), self)

		else: 
			self.notify_marooned_message(message)


from longtang.actors import messages

class CheckFileMetadata(messages.TraceableActorMessage):
	def __init__(self, source, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)

		self.__source = source

	def source(self):
		return self.__source

class FileMetadataIsComplete(messages.TraceableActorMessage):
	def __init__(self, source, metadata, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)	

		self.__source=source
		self.__metadata=metadata

	def source(self):
		return self.__source

	def metadata(self):
		return self.__metadata

class FileMetadataIsIncomplete(messages.TraceableActorMessage):
	def __init__(self, source, metadata, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)	

		self.__source=source
		self.__metadata=metadata

	def source(self):
		return self.__source

	def metadata(self):
		return self.__metadata

class WriteFileMetadata(messages.TraceableActorMessage):
	def __init__(self, source, metadata, tracking):
		messages.TraceableActorMessage.__init__(self,tracking)

		self.__source=source
		self.__metadata=metadata

	def source(self):
		return self.__source

	def metadata(self):
		return self.__metadata

class FileMetadataUpdated(messages.TraceableActorMessage):
	def __init__(self, source, metadata, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)	

		self.__source=source
		self.__metadata=metadata

	def source(self):
		return self.__source

	def metadata(self):
		return self.__metadata

class WriteFileMetadataFailed(messages.TrackingActorFailureMessage):
	def __init__(self, tracking, source_message, reason):
		messages.TrackingActorFailureMessage.__init__(self, tracking, reason, source_message)

	def source(self):
		return self.source_message().source()

	def metadata(self):
		return self.source_message().metadata()

class FileMetadataCouldNotBeChecked(messages.TraceableActorFailureMessage):
	def __init__(self, tracking, source_message, reason):
		messages.TraceableActorFailureMessage.__init__(self, tracking, reason, source_message)

	def source_file(self):
		return self.source_message().source()


import sys

class FileMetadata(object):

	def __init__(self):
		self.__artist=None
		self.__title=None
		self.__album=None
		self.__track_number=None

	def __capitalize_words(self, source_str):
		if source_str is not None:
			return ' '.join(word[0].upper() + word[1:] for word in source_str.lower().split())
		else:
			return source_str

	@property
	def artist(self):
		return self.__artist

	@artist.setter
	def artist(self, value):
		self.__artist = self.__capitalize_words(value)

	@property
	def title(self):
		return self.__title

	@title.setter
	def title(self, value):
		self.__title = self.__capitalize_words(value)

	@property
	def album(self):
		return self.__album

	@album.setter
	def album(self, value):
		self.__album = self.__capitalize_words(value)

	@property
	def track_number(self):
		return self.__track_number

	@track_number.setter
	def track_number(self, value):
		try:
			if value is not None:
				self.__track_number = int(value)
		except ValueError:
			#Potencially the value is not a number, we leave the tag empty
			pass 

	def missing_tags(self):
		if self.album is None:
			return True

		if self.artist is None:
			return True			

		if self.title is None:
			return True

		if self.is_track_number_missing():
			return True

		return False

	def is_track_number_missing(self):
		if self.track_number is None:
			return True

		return False		

	def __encode_value(self, value):
		if value is not None:
			return value.encode(sys.getdefaultencoding(),'replace')

		return value


	def __str__(self):
		return u'[artist: {0}, title: {1}, album: {2}, track: {3}]'.format(self.__artist, self.__title, self.__album, self.__track_number)					

class FileMetadataBuilder:
	
	def __init__(self):
		self.__artist=None
		self.__title=None
		self.__album=None
		self.__track_number=None

	def artist(self, artist):
		self.__artist = artist
		return self

	def title(self, title):
		self.__title = title
		return self

	def album(self, album):
		self.__album=album
		return self

	def track_number(self, track_number):
		self.__track_number=track_number
		return self

	def build(self):
		metadata = FileMetadata()

		metadata.artist=self.__artist
		metadata.title=self.__title
		metadata.album=self.__album
		metadata.track_number=self.__track_number

		return metadata

class FileMetadataBlender:
	
	def __init__(self):
		self.__from=None
		self.__with=None
		self.__override=False

	def from_base(self, metadata):
		self.__from = metadata
		return self

	def using(self, metadata):
		self.__with = metadata
		return self

	def override(self, value):
		self.__override=value
		return self

	def blend(self):

		builder = FileMetadataBuilder()

		artist = self.__with.artist if self.__from.artist is None or self.__override else self.__from.artist
		album = self.__with.album if self.__from.album is None or self.__override else self.__from.album
		title = self.__with.title if self.__from.title is None or self.__override else self.__from.title
		track_number = self.__with.track_number if self.__from.track_number is None else self.__from.track_number

		return builder.artist(artist).album(album).title(title).track_number(track_number).build()


from longtang.actors import messages

class LookupFileMetadata(messages.TraceableActorMessage):

	def __init__(self, source, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)
		self.__source=source

	def source(self):
		return self.__source


class FillUpMissingMetadataFields(messages.TraceableActorMessage):

	def __init__(self, source, metadata, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)
		self.__source=source
		self.__metadata=metadata

	def source(self):
		return self.__source

	def metadata(self):
		return self.__metadata

class FillUpMissingMetadataFieldsFailed(messages.TrackingActorFailureMessage):
	def __init__(self, tracking, source_message, reason):
		messages.TrackingActorFailureMessage.__init__(self, tracking, reason, source_message)

	def source_file(self):
		return self.source_message().source()

class FileMetadataSuccessfullyFilled(messages.TraceableActorMessage):

	def __init__(self, source, metadata, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)
		self.__source=source
		self.__metadata=metadata

	def source(self):
		return self.__source

	def metadata(self):
		return self.__metadata			

class FileMetadataFound(messages.TraceableActorMessage):

	def __init__(self, source, metadata, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)
		self.__metadata=metadata
		self.__source=source

	def metadata(self):
		return self.__metadata	

	def source(self):
		return self.__source

class FileMetadataNotFound(messages.TraceableActorMessage):
	def __init__(self, source, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)
		self.__source = source

	def source(self):
		return self.__source

class MusicBrainzServiceFailed(messages.TraceableActorMessage):
	def __init__(self, source, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)
		self.__source = source

	def source(self):
		return self.__source

class OverrideFileMetadata(messages.TraceableActorMessage):

	def __init__(self, source, metadata, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)
		self.__source=source
		self.__metadata=metadata

	def source(self):
		return self.__source

	def metadata(self):
		return self.__metadata

class OverrideFileMetadataDone(messages.TraceableActorMessage):

	def __init__(self, source, metadata, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)
		self.__source=source
		self.__metadata=metadata

	def source(self):
		return self.__source

	def metadata(self):
		return self.__metadata

class OverrideFileMetadataFailed(messages.TrackingActorFailureMessage):
	def __init__(self, tracking, source_message, reason):
		messages.TrackingActorFailureMessage.__init__(self, tracking, reason, source_message)

	def source_file(self):
		return self.source_message().source()		


import messages
import acoustid
import domain

from longtang.actors import actors
from longtang.common import domain as common_domain

class MusicMetadataActor(actors.Actor):

    __ACCOUSTID_API_KEY = '77CFDtEG'

    def receive(self, message):

        if isinstance(message, messages.FillUpMissingMetadataFields):

            self.logger().info(u'Filling up id3tag information .....', message.tracking())

            try:
                duration, fingerprint = acoustid.fingerprint_file(message.source())

                response = domain.AcoustidResponseWrapper(acoustid.lookup(self.__ACCOUSTID_API_KEY, fingerprint, duration, 'recordings releasegroups compress'))

                if response.successful() and response.found():

                    retrieved_metadata = response.extract_metadata()

                    self.logger().info(u'Metadata successfully retrieved from MusicBrainz. {0}'.format(retrieved_metadata), message.tracking())

                    blender = common_domain.FileMetadataBlender()
                    metadata = blender.from_base(message.metadata()).using(retrieved_metadata).blend()

                    if metadata.missing_tags():
                        self.logger().error('Could\'t find all the required id3tag information .....', message.tracking())
                        self.sender().tell(messages.FillUpMissingMetadataFieldsFailed(message.tracking(), message, 'Missing metadata fields could not be resolved'), self.myself())
                    else:
                        self.logger().info(u'Metadata blended. {0}'.format(metadata), message.tracking())
                        self.sender().tell(messages.FileMetadataSuccessfullyFilled(message.source(), metadata, message.tracking()), self.myself())
                else:
                    self.logger().error('Could\'t find all the required id3tag information .....', message.tracking())
                    self.sender().tell(messages.FillUpMissingMetadataFieldsFailed(message.tracking(), message, 'Missing metadata fields could not be resolved'), self.myself())

            except (acoustid.WebServiceError,acoustid.FingerprintGenerationError, KeyError) as e:
                self.sender().tell(messages.MusicBrainzServiceFailed(message.source(), message.tracking()), self.myself())

        elif isinstance(message, messages.LookupFileMetadata):

            self.logger().info('Looking up id3tag information .....', message.tracking())

            try:
                duration, fingerprint = acoustid.fingerprint_file(message.source())

                response = domain.AcoustidResponseWrapper(acoustid.lookup(self.__ACCOUSTID_API_KEY, fingerprint, duration, 'recordings releasegroups compress'))

                if response.successful() and response.found():

                    retrieved_metadata = response.extract_metadata()

                    self.logger().info(u'Metadata successfully retrieved from MusicBrainz. {0}'.format(retrieved_metadata), message.tracking())

                    self.sender().tell(messages.FileMetadataFound(message.source(), retrieved_metadata, message.tracking()), self.myself())
                else:
                    self.sender().tell(messages.FileMetadataNotFound(message.source(), message.tracking()), self.myself())

            except (acoustid.WebServiceError,acoustid.FingerprintGenerationError, KeyError) as e:
                self.sender().tell(messages.MusicBrainzServiceFailed(message.source()), self.myself())

        elif isinstance(message, messages.OverrideFileMetadata):

            self.logger().info(u'Overriding id3tag information .....', message.tracking())

            try:
                duration, fingerprint = acoustid.fingerprint_file(message.source())

                response = domain.AcoustidResponseWrapper(acoustid.lookup(self.__ACCOUSTID_API_KEY, fingerprint, duration, 'recordings releasegroups compress'))

                if response.successful() and response.found():

                    retrieved_metadata = response.extract_metadata()

                    self.logger().info(u'Metadata successfully retrieved from MusicBrainz. {0}'.format(retrieved_metadata), message.tracking())

                    blender = common_domain.FileMetadataBlender()
                    metadata = blender.from_base(message.metadata()).using(retrieved_metadata).override(True).blend()

                    if metadata.missing_tags():
                        self.logger().error('Could\'t find all the required id3tag information .....', message.tracking())
                        self.sender().tell(messages.OverrideFileMetadataFailed(message.tracking(), message, 'Missing metadata fields could not be resolved'), self.myself())
                    else:
                        self.logger().info(u'Metadata blended. {0}'.format(metadata), message.tracking())
                        self.sender().tell(messages.OverrideFileMetadataDone(message.source(), metadata, message.tracking()), self.myself())
                else:
                    self.logger().error('Could\'t find all the required id3tag information .....', message.tracking())
                    self.sender().tell(messages.OverrideFileMetadataFailed(message.tracking(), message, 'Missing metadata fields could not be resolved'), self.myself())

            except (acoustid.WebServiceError,acoustid.FingerprintGenerationError, KeyError) as e:
                self.sender().tell(messages.MusicBrainzServiceFailed(message.source(), message.tracking()), self.myself())

        else: 
            self.notify_marooned_message(message)                                                    


import unittest
import os

from longtang.actors import testutils
from longtang.actors.outbound.musicbrainz import musicbrainz, messages
from longtang.common import domain as common_domain
from hamcrest import *


class TestMusicBrainzActor(unittest.TestCase):

	def test_single_file_found(self):

		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')

		accoustic_tester = testutils.TestActorBuilder().withType(musicbrainz.MusicMetadataActor).build()

		source_file = os.path.join(data_dir, 'sample.mp3')

		accoustic_tester.tell(messages.LookupFileMetadata(source_file, 'tracking-id'))

		assert_that(accoustic_tester.inspector().num_instances(messages.FileMetadataFound), is_(equal_to(1)), 'Total amount of FileMetadataFound messages received is wrong')

		message = accoustic_tester.inspector().get_first(messages.FileMetadataFound)

		assert_that(message.source(), is_(equal_to(source_file)), 'Source file is not present in response')
		assert_that(message.metadata().artist, is_(equal_to('Bon Jovi')), 'Artist is not present within metadata')
		assert_that(message.metadata().album, is_(equal_to('New Jersey')), 'Album is not present within metadata')
		assert_that(message.metadata().title, is_(equal_to('Ride Cowboy Ride')), 'Album is not present within metadata')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id is missing')

	def test_file_not_found(self):
		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')

		accoustic_tester = testutils.TestActorBuilder().withType(musicbrainz.MusicMetadataActor).build()

		source_file = os.path.join(data_dir, 'not_found.mp3')

		accoustic_tester.tell(messages.LookupFileMetadata(source_file, 'tracking-id'))

		assert_that(accoustic_tester.inspector().num_instances(messages.FileMetadataNotFound), is_(equal_to(1)), 'Total amount of FileMetadataNotFound messages received is wrong')

		message = accoustic_tester.inspector().get_first(messages.FileMetadataNotFound)

		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id is missing')

	def test_override_metadata(self):

		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')

		accoustic_tester = testutils.TestActorBuilder().withType(musicbrainz.MusicMetadataActor).build()

		source_file = os.path.join(data_dir, '09-override_id3taginfo.mp3')		

		metadata_builder = common_domain.FileMetadataBuilder()
		metadata = metadata_builder.artist('My Test Artist').title('My Test Title').album('My Test Album').track_number('7').build()		

		accoustic_tester.tell(messages.OverrideFileMetadata(source_file, metadata, 'tracking-id'))		

		assert_that(accoustic_tester.inspector().num_instances(messages.OverrideFileMetadataDone), is_(equal_to(1)), 'Total amount of OverrideFileMetadataDone messages received is wrong')

		message = accoustic_tester.inspector().get_first(messages.OverrideFileMetadataDone)

		assert_that(message.source(), is_(equal_to(source_file)), 'Source file is not present in response')
		assert_that(message.metadata().artist, is_(equal_to('Bon Jovi')), 'Artist is not present within metadata')
		assert_that(message.metadata().album, is_(equal_to('New Jersey')), 'Album is not present within metadata')
		assert_that(message.metadata().title, is_(equal_to('Ride Cowboy Ride')), 'Album is not present within metadata')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id is missing')

	def test_override_metadata_failure(self):

		data_dir = os.path.join(testutils.current_dirpath(__file__), 'data')

		accoustic_tester = testutils.TestActorBuilder().withType(musicbrainz.MusicMetadataActor).build()

		source_file = os.path.join(data_dir, '09-override_id3taginfo.mp3')		

		metadata_builder = common_domain.FileMetadataBuilder()
		metadata = metadata_builder.artist('My Test Artist').title('My Test Title').album('My Test Album').track_number(None).build()		

		accoustic_tester.tell(messages.OverrideFileMetadata(source_file, metadata, 'tracking-id'))		

		assert_that(accoustic_tester.inspector().num_instances(messages.OverrideFileMetadataFailed), is_(equal_to(1)), 'Total amount of OverrideFileMetadataFailed messages received is wrong')

		message = accoustic_tester.inspector().get_first(messages.OverrideFileMetadataFailed)

		assert_that(message.source_file(), is_(equal_to(source_file)), 'Source file is not present in response')
		assert_that(message.tracking(), is_(equal_to('tracking-id')), 'Tracking id is missing')

if __name__ == '__main__':
    unittest.main()		


class Id3TagHandlerConfiguration():
	
	def __init__(self):
		self.__override_tags=None
		self.__offline_mode=None

	@property
	def override_tags(self):
		return self.__override_tags

	@override_tags.setter
	def override_tags(self, value):
		self.__override_tags = value

	@property
	def offline_mode(self):
		return self.__offline_mode

	@offline_mode.setter
	def offline_mode(self, value):
		self.__offline_mode = value

class Id3TagHandlerConfigurationBuilder():

	def __init__(self):
		self.__override_tags=False
		self.__offline_mode=False

	def override_tags(self, value):
		self.__override_tags=value
		return self

	def offline_mode(self, value):
		self.__offline_mode=value
		return self

	def build(self):
		config = Id3TagHandlerConfiguration()

		if self.__offline_mode and self.__override_tags:
			raise ValueError('Offline mode and Override tags are incompatible parameters. Only one of them should be enabled')

		config.override_tags=self.__override_tags
		config.offline_mode=self.__offline_mode

		return config


from longtang.actors import messages

class MediaFileAvailable(messages.TraceableActorMessage):
	def __init__(self, filepath, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)

		self.__filepath=filepath

	def filepath(self):
		return self.__filepath

class MediaFileHasBeenProcessed(messages.TraceableActorMessage):
	def __init__(self, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)

class MediaFileProcessingFailed(messages.TraceableActorFailureMessage):
	def __init__(self, tracking, source_message=None):
		messages.TraceableActorFailureMessage.__init__(self, tracking, "", source_message)		


import id3taghandler

from longtang.system import domain

class Id3TagSupervisorFactory(domain.ActorFactory):
	def __init__(self, config):
		self.__config = config

	def create(self, unique_id, system_ref):
		return id3taghandler.Id3TagSupervisor(self.__config, system_ref, unique_id)


import unittest
import os

from longtang.actors import testutils
from longtang.actors.logger import logger, messages, factory
from hamcrest import *


class TestLoggerActor(unittest.TestCase):

	def test_console_information_message(self):
		logger_tester = testutils.TestActorBuilder().withFactory(factory.LoggerActorFactory('DEBUG')).withId('logger-tester').build()

		logger_tester.tell(messages.FileInformationEventMessage('logger-tester','dummy.mp3','This is a simple information message'))

		assert_that(logger_tester.system().marooned_messages().reported_by('logger-tester'), is_(not_none()),'logger-tester actor marooned messages list is not none')
		assert_that(logger_tester.system().marooned_messages().reported_by('logger-tester').size(), is_(equal_to(0)),'logger-tester actor marooned messages list has elements')
		assert_that(logger_tester.system().marooned_messages().all(), is_(not_none()),'Global marooned messages list is not none')
		assert_that(logger_tester.system().marooned_messages().all().size(), is_(equal_to(0)),'Global marooned messages list is not empty')

	def test_console_error_message(self):
		logger_tester = testutils.TestActorBuilder().withFactory(factory.LoggerActorFactory('DEBUG')).withId('logger-tester').build()

		logger_tester.tell(messages.FileErrorEventMessage('logger-tester','dummy.mp3','This is a simple error message'))

		assert_that(logger_tester.system().marooned_messages().reported_by('logger-tester'), is_(not_none()),'logger-tester actor marooned messages list is not none')
		assert_that(logger_tester.system().marooned_messages().reported_by('logger-tester').size(), is_(equal_to(0)),'logger-tester actor marooned messages list has elements')
		assert_that(logger_tester.system().marooned_messages().all(), is_(not_none()),'Global marooned messages list is not none')
		assert_that(logger_tester.system().marooned_messages().all().size(), is_(equal_to(0)),'Global marooned messages list is not empty')

	def test_console_debug_message(self):
		logger_tester = testutils.TestActorBuilder().withFactory(factory.LoggerActorFactory('DEBUG')).withId('logger-tester').build()

		logger_tester.tell(messages.FileDebugEventMessage('logger-tester','dummy.mp3','This is a simple error message'))

		assert_that(logger_tester.system().marooned_messages().reported_by('logger-tester'), is_(not_none()),'logger-tester actor marooned messages list is not none')
		assert_that(logger_tester.system().marooned_messages().reported_by('logger-tester').size(), is_(equal_to(0)),'logger-tester actor marooned messages list has elements')
		assert_that(logger_tester.system().marooned_messages().all(), is_(not_none()),'Global marooned messages list is not none')
		assert_that(logger_tester.system().marooned_messages().all().size(), is_(equal_to(0)),'Global marooned messages list is not empty')		


from mutagen.easyid3 import EasyID3
from mutagen.id3 import TIT2, TALB, TRCK, TPE1
from mutagen.mp3 import MP3
from longtang.common import domain as common_domain

class ID3TagReaderFactory():

	@staticmethod
	def createFromType(source):
		if source.lower().endswith('mp3'):
			return Mp3ID3TagReader(source)

class ID3TagWriterFactory():

	@staticmethod
	def createFromType(source):
		if source.lower().endswith('mp3'):
			return Mp3ID3TagWriter(source)

class ID3TagReader():

	def artist(self):
		raise NotImplemented()

	def title(self):
		raise NotImplemented()

	def album(self):
		raise NotImplemented()

	def track_number(self):
		raise NotImplemented()

	def extract_metadata(self):

		builder = common_domain.FileMetadataBuilder()

		return builder.artist(self.artist()) \
						.album(self.album()) \
						.title(self.title()) \
						.track_number(self.track_number()).build()

class SimpleID3TagReader(ID3TagReader):

	#List all supported labels by mutagen: print EasyID3.valid_keys.keys()

	def __init__(self, source_file):
		self.__source_file = source_file

	def source_file(self):
		return self.__source_file

	def artist(self):
		return self.__read_tag('artist')

	def title(self):
		return self.__read_tag('title')

	def album(self):
		return self.__read_tag('album')

	def track_number(self):
		return self.__read_tag('tracknumber')

	def __read_tag(self, tagname):
		try:
			reader = EasyID3(self.__source_file)
			return reader[tagname][0]
		except KeyError:
			return None

class Mp3ID3TagReader(SimpleID3TagReader):

	def __init__(self, source_file):
		SimpleID3TagReader.__init__(self,source_file)

	def artist(self):
		return self.__read_tag('TPE1')

	def title(self):
		return self.__read_tag('TIT2')

	def album(self):
		return self.__read_tag('TALB')

	def track_number(self):
		return self.__read_tag('TRCK')

	def __read_tag(self, tagname):
		reader = MP3(self.source_file())
		reader.encoding=3 #utf8
		return reader[tagname].text[0] if tagname in reader else None

class ID3TagWriter():

	def save(metadata):
		raise NotImplemented()

class SimpleID3TagWriter(ID3TagWriter):
	def __init__(self, target_file):
		self.__target_file = target_file

	def target(self):
		return self.__target_file

	def save(self, metadata):

		writer = EasyID3(target_file)

		writer['artist'] = metadata.artist
		writer['title'] = metadata.title
		writer['album'] = metadata.album
		writer['tracknumber'] = metadata.track_number

		writer.save()

class Mp3ID3TagWriter(SimpleID3TagWriter):

	def __init__(self, target_file):
		SimpleID3TagWriter.__init__(self,target_file)

	def save(self, metadata):

		#reference: http://en.wikipedia.org/wiki/ID3

		audio_file = MP3(self.target())

		audio_file['TIT2'] = TIT2(encoding=3, text=[metadata.title]) #Title
		audio_file['TALB'] = TALB(encoding=3, text=[metadata.album]) #Album
		audio_file['TRCK'] = TRCK(encoding=3, text=[metadata.track_number]) #track
		audio_file['TPE1'] = TPE1(encoding=3, text=[metadata.artist]) #Artist

		audio_file.save()





from longtang.actors import messages

class PerformFilePackaging(messages.TraceableActorMessage):
	def __init__(self, source_file, metadata, target_dir, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)

		self.__source_file = source_file
		self.__metadata = metadata
		self.__target_dir = target_dir

	def source_file(self):
		return self.__source_file

	def metadata(self):
		return self.__metadata

	def target_dir(self):
		return self.__target_dir

class FilePackagingFinished(messages.TraceableActorMessage):

	def __init__(self, source_file, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)
		self.__source_file=source_file

	def source_file(self):
		return self.__source_file	

class FilePackagingFailure(messages.TraceableActorMessage):
	def __init__(self, source_file, metadata, reason, tracking):
		messages.TraceableActorMessage.__init__(self, tracking)
		self.__source_file=source_file
		self.__metadata=metadata
		self.__reason=reason

	def source_file(self):
		return self.__source_file

	def metadata(self):
		return self.__metadata

	def reason(self):
		return self.__reason


import messages
import domain

from longtang.actors import actors

class ID3TagWriterActor(actors.Actor):
	def receive(self, message):
		if isinstance(message, messages.WriteFileMetadata):

			self.logger().info(u'Saving metadata on file....{0}'.format(message.metadata()), message.tracking())

			if message.metadata().missing_tags():
				self.sender().tell(messages.WriteFileMetadataFailed(message.tracking(), message, 'One or more metadata field is missing'), self.myself())
			else:
				try:
					writer = domain.ID3TagWriterFactory.createFromType(message.source())
					writer.save(message.metadata())

					self.sender().tell(messages.FileMetadataUpdated(message.source(), message.metadata(), message.tracking()), self.myself())

				except IOError as e:

					self.logger().error(u'Metadata could not be saved. Reason: {0}'.format(str(e)), message.tracking())
					self.sender().tell(messages.WriteFileMetadataFailed(message.tracking(), message, str(e)), self.myself())
		else: 
			self.notify_marooned_message(message)


from longtang.common import mediatracking

class MetadataAvailable(mediatracking.MediaFileTrackingEvent):
	def __init__(self):
		mediatracking.MediaFileTrackingEvent.__init__(self, "METADATA_AVAILABLE", "Metadata is fully available")

class MetadataNotAvailable(mediatracking.MediaFileTrackingEvent):
	def __init__(self):
		mediatracking.MediaFileTrackingEvent.__init__(self, "METADATA_NOT_AVAILABLE_FAILURE", "Metadata could not be completed")

class MetadataEvaluationFailed(mediatracking.MediaFileTrackingEvent):
	def __init__(self):
		mediatracking.MediaFileTrackingEvent.__init__(self, "METADATA_EVALUATION_FAILURE", "Metadata could not be evaluated from file")		

class MetadataInspectionStarted(mediatracking.MediaFileTrackingEvent):
	def __init__(self):
		mediatracking.MediaFileTrackingEvent.__init__(self, "METADATA_INSPECTION_STARTED", "Metadata inspection from file started")

class FileMetadataIncomplete(mediatracking.MediaFileTrackingEvent):
	def __init__(self):
		mediatracking.MediaFileTrackingEvent.__init__(self, "FILE_METADATA_INCOMPLETE", "Metadata is incomplete. Trying MusicBrainz")

class MusicbrainzServiceFailed(mediatracking.MediaFileTrackingEvent):
	def __init__(self):
		mediatracking.MediaFileTrackingEvent.__init__(self, "MUSICBRAINZ_SERVICE_FAILURE", "Musicbrainz service is not responding")	



class ActivitySummary():

	def __init__(self, tracking_info):
		self.__tracking_info=tracking_info

	@staticmethod
	def create(tracking_info):
		return ActivitySummary(tracking_info)

	def total(self):
		return len(self.__tracking_info)

	def __is_failure(self, event):
		return 'FAILURE' in event.type()

	def __is_success(self, event):
		return not self.__is_failure(event)

	def __mediafile_failed(self, mediafile_tracking):
		for event in mediafile_tracking.tracking_events:
			if self.__is_failure(event):
				return True

		return False

	def total_failures(self):
		def map_elements(mediafile_tracking):
			if self.__mediafile_failed(mediafile_tracking):
				return 1

			return 0

		def reduce_elements(accum, value):
			return accum + value

		return reduce(reduce_elements,map(map_elements, self.__tracking_info.itervalues()))		

	def total_successes(self):
		def map_elements(mediafile_tracking):
			if self.__mediafile_failed(mediafile_tracking):
				return 0

			return 1

		def reduce_elements(accum, value):
			return accum + value

		return reduce(reduce_elements,map(map_elements, self.__tracking_info.itervalues()))

	def failures(self):
		for tracking_id, mediafile_tracking in self.__tracking_info.items():
			if self.__mediafile_failed(mediafile_tracking):
				yield tracking_id, mediafile_tracking

	def successes(self):
		for tracking_id, mediafile_tracking in self.__tracking_info.items():
			if not self.__mediafile_failed(mediafile_tracking):
				yield tracking_id, mediafile_tracking
